;MLC (Multi-platform Language for Calculators) by Epic Programming Studios
;TI-86 interpreter
;Written by Donald Straney (burntfuse) - straney@umbi.umd.edu

;Token parse/exec code

;//////////////////////////////Main loop//////////////////////////////

exec:
exec_reset:
	ld hl,(start_addr)					;get the address of the function MAIN to find where to start,
	ld (parse_ptr),hl
	ld (exec_sp),sp						;and save sp in case a command needs to be aborted
exec_loop:
	ld hl,(parse_ptr)
	ld a,(hl)							;look at the next token
	cp eof_token						;if it's the end of the file,
	ret z								;then we're done!
	ld de,exec_loop						;(push the address where all the handlers should return when done)
	push de
	ld de,expect_eol
	push de
	cp int_name_token					;if the line holds an expression,
	jp z,exec_expr_int					;then evaluate it!
	cp array_name_token
	jp z,exec_expr_array
	cp str_name_token
	jp z,exec_expr_str
	cp spr_name_token					;branch if it's changing the parameters of some sprite
	jp z,exec_spr_op
	cp ptr_name_token					;or a pointer
	jp z,exec_ptr_op
	cp map_name_token					;or even a tilemap
	jr z,exec_expr_map
	inc hl
	ld (parse_ptr),hl
exec_command:							;otherwise, it holds a command, so execute it
	add a,a								;find the address of the handler code depending on the command
	ld e,a
	ld d,0
	ld hl,exec_handler_table
	add hl,de
	ld a,(hl)
	inc hl
	ld h,(hl)
	ld l,a
	jp (hl)								;and execute the handler

;//////////////////////////////Expression handlers//////////////////////////////

exec_expr_map:
	call read_var_addr					;of course, get the map's address first
exec_expr_map_:
	ld a,(hl)
	cp index_start_token				;then find if it's a full map definition or just a definition for a single element
	jp nz,exec_expr_map_full
exec_expr_map_element:
	inc hl
	ld (parse_ptr),hl
	call get_var						;(make sure that the map has already been created)
	ld a,h
	or l
	jp z,err_noarray
	ex de,hl
	call parse_find_map_element			;parse the map index and find the address of the map element
	push hl
	call get_bmp						;get the address of the bitmap that should be stored to the map
	pop hl
	ld (hl),c							;and store it
	inc hl
	ld a,b
	and $0f
	ld (hl),a
	ret
exec_expr_map_full:
	push hl
	call get_var						;make sure the map's been created already
	ld a,h
	or l
	jp z,err_noarray
	inc hl
	inc hl
	ld e,(hl)							;and get its size
	inc hl
	ld d,(hl)
	inc hl
	push hl
	ex de,hl
	call _htimesl
	ld b,h
	ld c,l								;bc=length counter
	pop hl								;hl=map data
	pop de								;de=map data to parse
exec_expr_map_full_loop:
	ld a,(de)							;REPLACE W/GET_BMP LATER?
	cp eol_token
	jr z,exec_expr_map_full_end_loop
	inc de
	ld a,(de)							;store the bitmap in the tilemap
	ld (hl),a
	inc hl
	inc de
	ld a,(de)
	and $0f								;and set it to transparent
	ld (hl),a
	inc hl
	inc de
	dec bc								;decrement the length counter
	jr exec_expr_map_full_loop			;and loop
exec_expr_map_full_end_loop:
	ld a,b								;when done, make sure that the definition lengths match
	or c
	jp nz,err_arraydeflen
	ld (parse_ptr),de					;and update the parse pointer
	ret


exec_ptr_op:
	call read_var_addr					;get the pointer's address
	ld a,(hl)							;now...is the pointer being assigned a var?
	cp Lcolon
	jr nz,exec_ptr_op_expr
	inc hl
	ld b,(hl)							;if so, find the var/sprite/bitmap type
	inc hl
	ld e,(hl)
	inc hl
	ld d,(hl)
	inc hl
	ld (parse_ptr),hl
	ld a,b
	call tok_to_char					;and get the appropriate type char for it
	ld a,b
	ex de,hl
	ld de,(curr_var)
	jp set_ptr							;and set the pointer
exec_ptr_op_expr:
	ld (parse_ptr),hl					;if it's an expression that starts with a pointer,
	call get_ptr						;get the pointer type to determine where to go next!
	ld (curr_var),hl					;(set curr_var so that the var assigned to the pointer will be operated on by the expression handlers)
	ld hl,(parse_ptr)
	or a
	jp z,err_noptr
	cp int_char
	jp z,exec_expr_int_
	cp array_char
	jp z,exec_expr_array_
	cp string_char
	jr z,exec_expr_str_
	cp sprite_char
	jp z,exec_spr_op_
	cp map_char
	jp z,exec_expr_map_
	jp err_syntax



exec_expr_str:							;if it's a string var,
	call read_var_addr					;get the var index and save it for later
exec_expr_str_:
	ld a,(hl)							;see if the user is trying to assign a literal string
	inc hl
	cp int_char							;or a converted number to the string var
	jr z,exec_expr_str_num
exec_expr_str_str:
	call get_str						;evaluate the string expression
	ld de,(curr_var)					;then find the starting address of the string var in memory
	call _strcopy						;copy it to the end of the current string in the string var
	ld hl,(curr_var)					;before returning, make sure that the result of the string operations hasn't overflowed
	ex de,hl
	or a
	sbc hl,de
	ld de,str_max_len+1
	call cphlde
	jp nc,err_strlen
	ret
exec_expr_str_num:
	call get_expr_val_hl				;get the value of the numerical expression
	ld hl,(expr_sum)
	call format_num						;then format it to a displayable string
	ld de,(curr_var)					;find the storage location for the current var
	jp _strcopy							;and copy the formatted number to there


exec_expr_int:							;go here if it's an integer var
	call read_var_addr
exec_expr_int_:
	ld a,(hl)
	cp LplusSign						;WAIT! is this a var increment or var decrement line???
	jr z,exec_expr_inc					;if so, handle that separately
	cp Ldash
	jr z,exec_expr_dec
	cp Lasterisk
	jr z,exec_expr_selfop
	cp Lslash
	jr z,exec_expr_selfop
	call get_expr_val_hl				;now parse and evaluate the rest of the expression	ld hl,(expr_sum)					;get the sum of the expression
	ld hl,(expr_sum)
	ld de,(curr_var)					;and the index of the var being set
	jp set_var							;so that we can give that var the proper value!
exec_expr_inc:
	inc hl
	ld a,(hl)
	cp eol_token
	jr nz,exec_expr_selfop_alt
	ld (parse_ptr),hl
	push de
	call get_var						;[var]+, so increment var
	inc hl
	pop de
	jp set_var
exec_expr_dec:
	inc hl
	ld a,(hl)
	cp eol_token
	jr nz,exec_expr_selfop_alt
	ld (parse_ptr),hl
	push de
	call get_var
	dec hl								;decrement var
	pop de
	jp set_var
exec_expr_selfop_alt:
	dec hl
exec_expr_selfop:
	ld (parse_ptr),hl
	push de
	call get_var
	ld (expr_sum),hl
	call get_expr_val_loop
	ld hl,(expr_sum)
	pop de
	jp set_var


exec_expr_array:						;if it's an array,
	call read_var_addr
exec_expr_array_:
	ld (parse_ptr),hl					;update the parse pointer
	ld a,(hl)
	cp index_start_token				;check if it's an entire array definition or just defining a single element
	jr nz,exec_expr_array_whole
exec_expr_array_element:
	ld de,(curr_var)
	call get_var
	ex de,hl
	call parse_find_array_element		;find the location of the array element to be modified,
	push hl
	call get_expr_val					;then get the value that the array element is supposed to be set to,
	pop hl
	ld de,(expr_sum)					;and set it!
	ld (hl),e
	inc hl
	ld (hl),d
	ret									;so now we're done
exec_expr_array_whole:
	ld de,(curr_var)					;now get the address of the array var data
	call get_var
	ld a,h								;(make sure that it's been defined)
	or l
	jp z,err_noarray
	ld a,(hl)							;(save array length)
	ld (exec_expr_array_length),a
	inc hl
	ld b,0								;init array length counter
exec_expr_array_whole_loop:
	push bc
	push hl
	call get_expr_val					;get the value of this element of the array
	pop hl
	pop bc
	ld de,(expr_sum)					;and store it to the array
	ld (hl),e
	inc hl
	ld (hl),d
	inc hl
	inc b								;and increment index counter
	push hl
	ld hl,(parse_ptr)
	ld a,(hl)
	cp eol_token
	pop hl
	jr nz,exec_expr_array_whole_loop	;if it's not the end of the line, then loop again
	ld a,b								;when done, make sure that the literal array and the array var
exec_expr_array_length	equ	$+1
	cp 0								;have the same length!
	jp nz,err_arraydeflen
	ret									;if they do, then go back to execute the next line


exec_spr_op:
	call read_var_addr					;get the index of the sprite we're going to modify
exec_spr_op_:
	ld a,(hl)
	cp LcapB
	jr z,exec_obj_op_bmp
	push af								;and the parameter that gets modified
	inc hl
	call get_expr_val_hl				;now get the value that the parameter should be set to
	pop af								;and set it, depending on which parameter it is
	ld de,(curr_var)
	ld bc,(expr_sum)
	sub LcapD				;DX (D)
	jp z,set_spr_dx
	dec a					;DY (E)
	jp z,set_spr_dy
	sub LcapV-LcapE			;VX (V)
	jp z,set_spr_vx
	dec a					;VY (W)
	jp z,set_spr_vy
	push af
	ld hl,(expr_sum)
	call format_spr_pos
	pop bc
	dec b					;X (X)
	jp z,set_spr_x
	jp set_spr_y
exec_obj_op_bmp:						;the user's trying to set the sprite's bitmap property...
	inc hl
	call get_bmp_hl
	ld de,(curr_var)
	jp set_spr							;and finally, set the original sprite


get_bmp:								;get bitmap at (parse_ptr), return address in bc, parse_ptr updated
	ld hl,(parse_ptr)
get_bmp_hl:
	ld a,(hl)							;is it a bitmap name or another sprite's bitmap?
	inc hl
	ld e,(hl)
	inc hl
	ld d,(hl)
	inc hl
	ld (parse_ptr),hl
	cp bmp_name_token
	jr z,get_bmp_bmp
	cp ptr_name_token
	jr z,get_bmp_ptr
	cp map_name_token
	jr z,get_bmp_map
get_bmp_spr:							;if it's the bitmap of another sprite,
	call get_spr						;using the sprite's address, find its bitmap's address,
	ld hl,(parse_ptr)
	inc hl
	ld (parse_ptr),hl
	ld b,d
	ld c,e								;and return it in bc
	ret
get_bmp_ptr:							;if it's a pointer,
	call get_ptr
	cp bitmap_char						;look at the data type
	jr z,get_bmp_ptr_bmp
	cp sprite_char
	jp nz,err_bmp
get_bmp_ptr_spr:
	ld de,(parse_ptr)					;if it's a sprite, make sure that there's a '.BMP' after it
	ld a,(de)
	cp LcapB
	jp nz,err_bmp
	inc de
	ld (parse_ptr),de
	ex de,hl
	call get_spr						;then get the bitmap
	ld b,d								;and return it in bc
	ld c,e
	ret
get_bmp_ptr_bmp:						;if it's a pointer, then get its address
	ld c,l
	ld b,h
	ret
get_bmp_bmp:							;if it's a bitmap name, we can get off easy!
	ld b,d								;just transfer the address to bc
	ld c,e
	ret
get_bmp_map:
	ld a,(hl)
	cp index_start_token
	jp nz,err_bmp
	inc hl
	call parse_find_map_element_hl		;parse the map index
	ld c,(hl)							;and get the address of the bitmap from the tilemap data
	inc hl
	ld a,(hl)
	or $a0
	ld b,a
	ret


get_expr_val:							;get value of expression at (parse_ptr), parse_ptr updated, value returned in expr_sum, c set if new line
	ld hl,(parse_ptr)
get_expr_val_hl:
	call get_val						;init the sum of the expression depending on whether it starts out with a varname or number
	ld (expr_sum),hl
get_expr_val_loop:
	ld hl,(parse_ptr)
	ld a,(hl)							;get the next char in the expression
	inc hl
	sub Lasterisk						;otherwise, check for operators
	jr z,get_expr_val_m
	dec a
	jr z,get_expr_val_a
	sub Ldash-LplusSign
	jr z,get_expr_val_s
	sub Lslash-Ldash
	jr z,get_expr_val_d
	sub Lbar-Lslash
	jr z,get_expr_val_mod
	jr get_expr_val_end
get_expr_val_a:							;	+
	call get_val						;get the value of the next var/num,
	ld de,(expr_sum)
	add hl,de							;add it to the current sum,
	ld (expr_sum),hl
	jr get_expr_val_loop				;and loop
get_expr_val_s:							;	-
	call get_val
	ld de,(expr_sum)
	ex de,hl
	or a
	sbc hl,de
	ld (expr_sum),hl
	jr get_expr_val_loop
get_expr_val_m:							;	*
	call get_val						;ooohh...we have to do a 16-bit multiply here...this'll be hard...
	ld de,(expr_sum)
	call mult_num
	ld (expr_sum),hl
	jr get_expr_val_loop
get_expr_val_d:							;	/
	call get_val						;and a 16-bit divide too...why am I always stuck with the hard ones???
	ld de,(expr_sum)
	ex de,hl
	call div_num
	ld (expr_sum),hl
	jr get_expr_val_loop
get_expr_val_mod:						; x | y
	call get_val
	ld de,(expr_sum)
	ex de,hl							;hl=x, de=y
	push hl
	push de
	call div_num						;hl=x/y
	pop de
	call mult_num						;hl=(x/y)*y
	pop de
	ex de,hl							;hl=x, de=(x/y)*y
	or a
	sbc hl,de							;hl=x-(x/y)*y
	ld (expr_sum),hl
	jr get_expr_val_loop
get_expr_val_end:
	dec hl
	ld (parse_ptr),hl					;point to the beginning of the next line
	ret


get_val:								;parse number or varname at (hl), parse_ptr updated, number returned in hl
	ld a,(hl)
	inc hl
	ld e,(hl)							;first get the address
	inc hl
	ld d,(hl)
	inc hl
	ld (parse_ptr),hl
	cp lit_num_token					;is it a number?
	jp z,get_val_num					;if so, parse that, obviously
	cp spr_name_token					;it's also accepted if it's a sprite's parameter...
	jr z,get_val_spr
	cp ptr_name_token					;or a pointer!!!
	jr z,get_val_ptr
get_val_var:
	cp int_name_token					;is it a var?
	jp z,get_var						;if so, parse that
	cp array_name_token
	jp nz,err_num
get_val_array:							;if it's an element of an array, then get the value of that element
	call get_var
	ld de,(expr_sum)					;FIRST, SAVE THE CURRENT EXPRESSION'S SUM!!!
	push de
	ex de,hl
	call parse_find_array_element		;parse the array index and find the correct element
	call ldhlind						;and get the array element's value from that
	pop de
	ld (expr_sum),de					;then restore current expression sum
	ret
get_val_ptr:
	call get_ptr						;hmmm...what type of pointer is it?
	ex de,hl
	ld hl,(parse_ptr)					;(get address of var in de, parse_ptr in hl)
	cp sprite_char
	jr z,get_val_spr
	cp int_char
	jp z,get_var
	cp array_char
	jp z,get_val_array
	or a
	jp z,err_noptr
	jp err_num
get_val_spr:
	ld a,(hl)							;check on the sprite parameter
	inc hl
	ld (parse_ptr),hl					;and update pointer
	sub LcapD
	jr z,get_val_obj_dx
	dec a
	jr z,get_val_obj_dy
	sub LcapV-LcapE
	jr z,get_val_obj_vx
	dec a
	jr z,get_val_obj_vy
	dec a
	jr z,get_val_obj_x
get_val_obj_y:							;if it's the y coordinate,
	call get_spr						;get all the sprite's parameters,
	ld h,0
	ld l,c								;but save only the y pos
	ld a,c
	cp 128
	ret c
	dec h
	ret
get_val_obj_x:
	call get_spr
	ld h,0
	ld l,b
	ld a,b
	cp 128
	ret c
	dec h
	ret
get_val_obj_vy:
	call get_spr_vy
	ld l,c
	ld h,b
	ret
get_val_obj_vx:
	call get_spr_vx
	ld l,c
	ld h,b
	ret
get_val_obj_dx:
	call get_spr_dx
	ld l,c
	ld h,b
	ret
get_val_obj_dy:
	call get_spr_dy
	ld l,c
	ld h,b
	ret
get_val_num:
	ex de,hl							;if the expression is a number, get it in hl
	ret


get_str:								;get string at (hl), return pointer in hl, update parse_ptr
	ld de,str_temp_mem
	ld (str_temp_ptr),de
get_str_loop:
	ld a,(hl)							;look at the next token
	sub lit_str_token					;if it's a string,
	jr z,get_str_lit					;then just return a pointer to that
	dec a
	jr z,get_str_var_num				;if it's a number, format it to a string
	sub 3
	jr z,get_str_var_num
	sub 2
	jr z,get_str_ptr					;if it's a pointer, treat it like a var
	dec a
	jr z,get_str_var_str
	dec a
	jr z,get_str_var_num
get_str_var_num:
	call get_val						;get the value of the num/var/whatever...
	call format_num						;then we have to format it...*sigh*...
	jr get_str_end
get_str_var_str:
	inc hl
	ld e,(hl)
	inc hl
	ld d,(hl)
	inc hl
	ld (parse_ptr),hl
get_str_var_str_:
	ex de,hl
	jr get_str_end
get_str_ptr:
	push hl
	inc hl
	ld e,(hl)
	inc hl
	ld d,(hl)
	inc hl
	ld (parse_ptr),hl
	call get_ptr						;make sure that the pointer's pointing to a string var...
	ex de,hl							;and put the string's address in de...
	pop hl
	or a
	jp z,err_noptr
	cp string_char						;the rest can be handled by the existing code! :-)
	jr z,get_str_var_str_
	cp int_char
	jr z,get_str_var_num
	cp array_char
	jr z,get_str_var_num
	cp sprite_char
	jr z,get_str_var_num
	jp err_str
get_str_lit:
	inc hl								;get past the literal string token, and we have the zero-terminated string right there!!!
	push hl								;but we still have to update the parse pointer...save the starting address of the string
	ld bc,str_max_len+1
	xor a								;search for the end of the string
	cpir
	ld (parse_ptr),hl					;and update the parse pointer
	pop hl								;now restore the pointer to the string
get_str_end:
	ld de,(str_temp_ptr)				;copy the string to a temp area
	call _strcopy
	ld (str_temp_ptr),de
	ld hl,(parse_ptr)
	ld a,(hl)
	cp LplusSign
	jr nz,get_str_ret
	inc hl
	jp get_str_loop
get_str_ret:
	xor a
	ld (de),a
	ld hl,str_temp_mem
	ret


;//////////////////////////////Signed Math Routines//////////////////////////////

mult_num:					;hl=hl*de
	call is_hl_neg
	jp nc,mult_num_hl
	call is_de_neg
	jp nc,mult_num_neg_de
	jp mult16
mult_num_hl:
	call is_de_neg
	jp c,mult16
mult_num_neg_de:
	call neg_de
	call mult16
	jp neg_hl

div_num:					;hl=hl/de
	call is_hl_neg
	jp nc,div_num_hl
	call is_de_neg
	jp nc,div_num_neg_de
	jp div16
div_num_hl:
	call is_de_neg
	jp nc,div_num_neg_both
div_num_neg_hl:
	call neg_hl
	jp div_num_neg
div_num_neg_de:
	call neg_de
div_num_neg:
	call div16
	jp neg_hl
div_num_neg_both:
	call neg_hl
	call neg_de
	jp div16

is_hl_neg:					;nc if hl is neg, c if pos
	ld a,h
	cp $80
	ret

is_de_neg:
	ld a,d
	cp $80
	ret

neg_de:						;de= -de
	ld a,d
	cpl
	ld d,a
	ld a,e
	cpl
	ld e,a
	inc de
	ret

neg_hl:
	ld a,h
	cpl
	ld h,a
	ld a,l
	cpl
	ld l,a
	inc hl
	ret

cp_num:						;cp hl,de
	call cphlde
	ret z
	call is_hl_neg
	jp nc,cp_num_hl_neg
	call is_de_neg
	jp nc,cp_num_ccf
	jp cphlde
cp_num_hl_neg:
	call is_de_neg
	jp nc,cphlde
	scf
	ret
cp_num_ccf:
	or a
	ret

;//////////////////////////////Handler fixed-address routines//////////////////////////////

exec_c_line_draw:
	push hl
	ld b,a
	ld c,0
exec_c_line_draw1	equ	$+1
	ld hl,0
exec_c_line_draw2	equ	$+1
	ld de,0
	call FLine
	pop hl
	ret

exec_c_pixl_draw1:
	call FindPixel
	ld b,a
	ld de,gray1_buf
	add hl,de
	ret
exec_c_pixl_draw2:
	ld a,b
	ld de,gray2_buf-gray1_buf
	add hl,de
	ret

exec_c_pixt_lgray:
	ld hl,expr_sum
	inc (hl)
	ret
exec_c_pixt_dgray:
	ld hl,expr_sum
	inc (hl)
	inc (hl)
	ret

exec_c_rect_fill:
	ld bc,(temp_buf)
	ld de,(temp_buf+2)
	inc b
	inc c
	push hl
	call box
	pop hl
	ret
exec_c_rect_lines:
	ld b,a
	ld c,0
	push hl
	ld hl,(temp_buf)	;x1,y1 - x1,y2
	ld a,(temp_buf+1)
	ld d,a
	ld a,(temp_buf+2)
	ld e,a
	call FLine
	ld hl,(temp_buf)	;x1,y1 - x2,y1
	ld a,(temp_buf+3)
	ld d,a
	ld a,(temp_buf)
	ld e,a
	call FLine
	ld hl,(temp_buf+2)	;x2,y2 - x1,y2
	ld a,(temp_buf+1)
	ld d,a
	ld a,(temp_buf+2)
	ld e,a
	call FLine
	ld hl,(temp_buf+2)	;x2,y2 - x2,y1
	ld a,(temp_buf+3)
	ld d,a
	ld a,(temp_buf)
	ld e,a
	call FLine
	pop hl
	ret

exec_c_disp_set:
	push de
	push de
	call get_spr_vx						;and the amount to add to the coordinates for the velocity...
	push bc
	call get_spr_dx
	pop hl
	add hl,bc							;update the position from that
	ld b,h
	ld c,l
	call set_spr_dx
	pop de
	call get_spr_vy
	push bc
	call get_spr_dy
	pop hl
	add hl,bc
	ld b,h
	ld c,l
	call set_spr_dy
	pop de
	ret


exec_c_clsn_all:
	call expect_num						;make sure that it's a .0
	ld a,h
	or l
	jp nz,err_invarg
	call expect_intname					;then store the result var as usual...
	ld (curr_var),de
	call expect_ptrname					;and the result pointer
	ld (exec_c_clsn_ptr),de
	ld a,(num_sprites)					;now get the number of sprites we'll have to test
	ld b,a								;(there can't be 0 since the first sprite has to be one of them)
	pop hl								;restore the position of the first sprite
	ld (exec_c_clsn_all_pos),hl
	ld de,spr_1_data					;and init the pointer at the beginning of the sprite table
exec_c_clsn_all_loop:
	push de
	push bc
exec_c_clsn_nosprite	equ	$+1
	ld hl,0
	call cphlde						;if it's the base sprite, skip it
	jr z,exec_c_clsn_all_skip
	call get_spr						;otherwise, find the next sprite in the table
	ld (exec_c_clsn_sprite2),de
	call get_bmp_size
	ld (exec_c_clsn_spr2size),a			;get its size
	ld h,b								;and position
	ld l,c
exec_c_clsn_all_pos		equ	$+1
	ld de,0
	call exec_c_clsn_test				;and test it against the base sprite
	jr c,exec_c_clsn_all_yes			;if it's touching, then end the loop and set the var
exec_c_clsn_all_skip:
	pop bc								;if it isn't touching,
	pop de
	ld hl,spr_data_len
	add hl,de							;then move to the next sprite
	ex de,hl
	djnz exec_c_clsn_all_loop			;and loop
	jp exec_c_clsn_no					;if it got to here, then there must be no sprites touching it
exec_c_clsn_all_yes:
	pop bc
	pop hl								;set the pointer to point to the sprite that it's touching
exec_c_clsn_ptr	equ	$+1
	ld de,0
	ld a,sprite_char
	call set_ptr
	jp exec_c_clsn_yes
exec_c_clsn_test:						;de=spr1 pos, hl=spr2 pos, c set if collision
exec_c_clsn_spr1size	equ	$+1
	ld c,0
exec_c_clsn_spr2size	equ	$+1
	ld b,0
	ld a,h								;check if the sprites overlap
	add a,b		;x2+8>x1
	cp d
	jp c,exec_c_clsn_test_no
	ld a,d
	add a,c		;x1+8>x2
	cp h
	jp c,exec_c_clsn_test_no
	ld a,l
	add a,b		;y2+8>y1
	cp e
	jp c,exec_c_clsn_test_no
	ld a,e
	add a,c		;y1+8>y2
	cp l
	jp c,exec_c_clsn_test_no
	ld a,b
	cp 16
	jp z,exec_c_clsn_test_yes
	ld a,c
	cp 16
	jp z,exec_c_clsn_test_yes
	xor a
	ld (exec_c_clsn_lr),a
	ld a,h								;compare sprite x positions
	cp d
	jr c,exec_c_clsn_endtestx
	ex de,hl
	ld a,$eb ;ex de,hl					;(0 - sprite 1 (hl) is left, sprite 2 (de) is right, 1 - sprite 2 (de) is left, sprite 1 (hl) is right)
	ld (exec_c_clsn_lr),a
exec_c_clsn_endtestx:					;hl=left sprite pos, de=right sprite pos
	ld a,d
	sub h								;a=amount to shift left sprite left
	ld (exec_c_clsn_sl),a
	ld b,0
	ld a,l								;now compare sprite y positions
	cp e
	jr c,exec_c_clsn_endtesty
	ex de,hl
	inc b								;(0 - sprite 1 (hl) is top, sprite 2 (de) is bottom, 1 - sprite 2 (de) is top, sprite 1 (hl) is bottom)
exec_c_clsn_endtesty:					;hl=top sprite pos, de=bottom sprite pos
	ld a,e
	sub l								;a=downwards offset for top sprite
	ld c,a
	ld a,8
	sub c
	ld (exec_c_clsn_rows),a				;calculate the number of rows to test from that
	ld a,c
	djnz exec_c_clsn_setoffset1			;if there's a difference in height, find which sprite to offset
exec_c_clsn_setoffset2:
	ld b,a
	xor a
	jr exec_c_clsn_endoffset
exec_c_clsn_setoffset1:
	ld b,0
exec_c_clsn_endoffset:					;offset the correct sprite...
	ld (exec_c_clsn_offset1),a
	ld a,b
	ld (exec_c_clsn_offset2),a
exec_c_clsn_sprite2	equ	$+1
	ld hl,0
	ld bc,8*2+1
	add hl,bc
exec_c_clsn_offset2	equ	$+1
	ld c,0
	ld b,0
	add hl,bc
	ex de,hl
exec_c_clsn_sprite1	equ	$+1
	ld hl,0
	ld bc,8*2+1
	add hl,bc							;hl=sprite 1, de=sprite 2
exec_c_clsn_offset1	equ	$+1
	ld c,0
	ld b,0
	add hl,bc
exec_c_clsn_lr	equ	$
	nop									;switch them if necessary
exec_c_clsn_rows	equ	$+1
	ld b,8
	ld a,b								;if there aren't any overlapping rows,
	or a
	jp z,exec_c_clsn_no					;then the sprites must not be overlapping!
exec_c_clsn_loop:
	push bc								;otherwise, save the number of rows left to test
	ld a,(hl)
	cpl									;then get the mask byte from the left sprite
	ld c,a
exec_c_clsn_sl	equ	$+1
	ld b,0								;and shift it the correct number of times
	ld a,b
	or a
	jr z,exec_c_clsn_loop_noshift
exec_c_clsn_shiftloop:
	sla c
	djnz exec_c_clsn_shiftloop
exec_c_clsn_loop_noshift:
	ld a,(de)							;then overlay it with the mask byte from the right-hand sprite
	cpl
	and c								;the big test...
	pop bc
	jr nz,exec_c_clsn_test_yes			;if there's an overlap, there's a collision - set the var to 1
	inc hl
	inc de
	djnz exec_c_clsn_loop				;otherwise, move on to the next row
exec_c_clsn_test_no:					;if all the rows have been tested already, none must be overlapping, so set the var to a 0
	xor a
	ret
exec_c_clsn_test_yes:
	scf
	ret
exec_c_clsn_yes:
	ld hl,1
	jr exec_c_clsn_end
exec_c_clsn_no:
	ld hl,0
exec_c_clsn_end:
	ld de,(curr_var)					;and set the var accordingly
	jp set_var_warray


file_pos	equ	temp_buf
new_size	equ	temp_buf+2
var_type	equ	temp_buf+4

exec_c_wrte_create_new_dest:
	rst 20h
	ld hl,(new_size)
	call _createstrng
	call _ex_ahl_bde
	call _ahl_plus_2
	jp _set_abs_dest_addr
exec_c_wrte_set_src_copy:
	call exec_c_wrte_get_var_start
	call _set_abs_src_addr
	ex de,hl
	call _data_size_to_de
	ex de,hl
	xor a
	jp _mm_ldir_set_size
exec_c_wrte_del:
	rst 20h
	rst 10h
	jp _delvar
exec_c_wrte_get_var_start:
	rst 20h
	rst 10h
	call _ex_ahl_bde
	jp _ahl_plus_2


get_cos:
	ld de,360
get_cos_stillneg:
;T=T+360
	add hl,de
;IF T<0
	call is_hl_neg							;(compensate for negative angles)
;	FGOB
	jr nc,get_cos_stillneg
;T=T|360
	push hl
	push de
	call div16
	pop de
	call mult16
	pop de
	ex de,hl
	or a
	sbc hl,de								;hl=T|360
;TS=T
	ld (ts_save),hl
;T=T|90
	ld de,90
	push hl
	push de
	call div16
	pop de
	call mult16
	pop de
	ex de,hl
	or a
	sbc hl,de								;hl=T|90
	ld (t_save),hl
;IF (TS/90)|2=1
	ld hl,(ts_save)
	ld de,90
	call div16								;hl=TS/90
	ld de,2
	push hl
	push de
	call div16
	pop de
	call mult16
	pop de
	ex de,hl
	or a
	sbc hl,de								;hl=(TS/90)|2
	dec hl									;(TS/90)|2 = 1?
	ld a,h
	or l
	ld hl,(t_save)
	jr nz,get_cos_skip
;	T=90-T
	ex de,hl
	ld hl,90
	or a
	sbc hl,de
get_cos_skip:
;T=3/23*T^2
	ld d,h
	ld e,l
	call mult_num							;hl=T^2
	ld de,3
	call mult16								;hl=3*T^2
	ld de,23
	call div16								;hl=3/23*T^2
;T=1000-T
	ex de,hl
	ld hl,1000
	or a
	sbc hl,de								;hl=1000-T
	ld (t_save),hl
;IF (TS+90)/180=1
	ld hl,(ts_save)
	ld de,90
	add hl,de
	ld de,180
	call div16
	dec hl
	ld a,h
	or l
	ld hl,(t_save)
	ret nz
;	T=-T
	jp neg_hl								;hl=-T


exec_c_cmap_get_tilenums:
exec_c_cmap_x	equ	$+1
	ld a,0
	sub b
	neg										;a=spr_x-map_x
	and %11111000
	rra
	rra
	rra										;a=(spr_x-map_x)/8
	ld (hl),a
	inc hl
	ld (hl),0
	inc hl
	ex de,hl
exec_c_cmap_y	equ	$+1
	ld a,0
	sub c
	neg										;a=spr_y-map_y
	and %11111000
	rra
	rra
	rra										;a=(spr_y-map_y)/8
	ld (hl),a
	inc hl
	ld (hl),0
	inc hl
	ex de,hl
	ret


shft_x	equ	temp_buf
shft_y	equ	temp_buf+2
shft_size	equ	temp_buf+4
shft_color	equ	temp_buf+6

exec_c_shft_y_pos_loop:
;de=end of buf
	push de
	ld hl,shft_buf-1
	ld bc,(shft_size)
	add hl,bc
	ex de,hl
	lddr
	pop de
	push hl
	ld hl,screen_len
	ld bc,(shft_size)
	or a
	sbc hl,bc
	ld b,h
	ld c,l
	pop hl
	lddr
	ex de,hl
	inc de
	ld hl,shft_buf
	ld bc,(shft_size)
	ldir
	ret
exec_c_shft_y_neg_loop:
;hl=start of buf
	push hl
	ld de,shft_buf
	ld bc,(shft_size)
	ldir
	pop de
	push hl
	ld hl,screen_len
	ld bc,(shft_size)
	or a
	sbc hl,bc
	ld b,h
	ld c,l
	pop hl
	ldir
	ex de,hl
	dec de
	ld hl,shft_buf-1
	ld bc,(shft_size)
	add hl,bc
	lddr
	ret
exec_c_shft_y_pos_fill:
;hl=end of buf
;a=color
	push hl
	ld bc,(shft_size)
	or a
	sbc hl,bc
	push hl
	ld hl,screen_len
	or a
	sbc hl,bc
	ld b,h
	ld c,l
	pop hl
	pop de
	lddr
	ld h,d
	ld l,e
	dec de
	ld (hl),a
	ld bc,(shft_size)
	dec c
	lddr
	ret
exec_c_shft_y_neg_fill:
;hl=start of buf
;a=color
	push hl
	ld bc,(shft_size)
	add hl,bc
	push hl
	ld hl,screen_len
	or a
	sbc hl,bc
	ld b,h
	ld c,l
	pop hl
	pop de
	ldir
	ld h,d
	ld l,e
	inc de
	ld (hl),a
	ld bc,(shft_size)
	dec c
	ldir
	ret


;//////////////////////////////Fixed handlers//////////////////////////////

exec_c_if:
	pop hl
	ld hl,(parse_ptr)					;branch depending on if strings or numbers are being compared
	ld a,(hl)
	inc hl
	cp int_char
	jr z,exec_c_if_num
	cp bitmap_char
	jr z,exec_c_if_bmp
exec_c_if_str:
	call get_str						;get a pointer to the first string
	ld de,str_temp_mem+str_max_len+1
	call _strcopy
	ld hl,(parse_ptr)					;and look at the operator between the strings
	ld a,(hl)
	inc hl
	push af
	call get_str						;now get a pointer to the second string
	ex de,hl
	call expect_eol						;(make sure that that's the end of the line)
	pop af
	ld hl,str_temp_mem+str_max_len+1
	cp Lexclam
	jr z,exec_c_if_str_ne
	cp LEQ
	jp nz,err_invop
exec_c_if_str_eq:
	call str_cmp						;compare the strings
	jp z,exec_c_if_yes
	jp exec_c_if_no
exec_c_if_str_ne:
	call str_cmp
	jp nz,exec_c_if_yes
	jp exec_c_if_no
exec_c_if_bmp:
	call get_bmp_hl						;get the address of the first bitmap
	push bc
	ld hl,(parse_ptr)
	ld a,(hl)							;save the operator between the bitmaps
	push af
	inc hl
	call get_bmp_hl						;and get the second
	call expect_eol
	ld h,b
	ld l,c								;hl=bmp2
	pop af								;a=operator
	pop de								;de=bmp1
	cp LEQ
	jr z,exec_c_if_bmp_eq
	cp Lexclam
	jp nz,err_invop
exec_c_if_bmp_ne:
	call cphlde							;are the bitmaps the same?
	jp nz,exec_c_if_yes
	jp exec_c_if_no
exec_c_if_bmp_eq:
	call cphlde
	jp z,exec_c_if_yes
	jp exec_c_if_no
exec_c_if_num:
	call get_expr_val_hl				;get the value of the first expression
	ld de,(expr_sum)
	push de
	ld a,(hl)							;then the operator
	push af
	inc hl
	call get_expr_val_hl				;then the value of the second expression
	pop af								;now restore operator and branch depending on what it is
	pop de
	ld hl,(expr_sum)					;(put value of first expression in de, second in hl)
	cp LLT
	jr z,exec_c_if_lt
	cp LGT
	jr z,exec_c_if_gt
	cp LEQ
	jr z,exec_c_if_eq
	cp Lexclam
	jr z,exec_c_if_ne
	jp err_invop						;if it isn't a valid operator, then show error
exec_c_if_lt:
	call cp_num
	jp c,exec_c_if_num_no
	jp z,exec_c_if_num_no
	jp exec_c_if_continue
exec_c_if_gt:
	call cp_num
	jp nc,exec_c_if_num_no
	jp exec_c_if_continue
exec_c_if_eq:
	call cphlde							;are they equal?
	jp z,exec_c_if_continue				;if so, execute the next line normally
	jp exec_c_if_num_no
exec_c_if_ne:
	call cphlde							;are they equal?
	jp z,exec_c_if_num_no
exec_c_if_continue:
	ld hl,(parse_ptr)					;is the expression done?
	ld a,(hl)
	cp eol_token
	jr z,exec_c_if_num_yes				;if so, return to execute the next line
	ld hl,(parse_ptr)
	ld a,(hl)
	cp Lampersand						;otherwise, check for an '&' between expressions
	jp nz,err_invop
	inc hl
	jp exec_c_if_num					;and evaluate the next part of the expression
exec_c_if_no:
	ld hl,(parse_ptr)
	jr exec_c_if_no_continue
exec_c_if_yes:
	ld hl,(parse_ptr)
	jr exec_c_if_yes_continue
exec_c_if_num_no:
	ld hl,(parse_ptr)
	call goto_eol
	inc hl
exec_c_if_no_continue:
	ld a,(hl)
	cp then_token
	jp nz,find_eol_hl					;if there isn't a THEN block, just skip the line
exec_c_if_no_loop:						;otherwise, find the end of the block
	call goto_eol
	inc hl
	ld a,(hl)
	cp ifnd_token
	jp nz,exec_c_if_no_loop
	jp find_eol_hl
exec_c_if_num_yes:
	inc hl
	ld (parse_ptr),hl
exec_c_if_yes_continue:
	ld a,(hl)
	cp then_token						;is it a THEN block following?
	ret nz								;if not, just return
	inc hl
	ld (parse_ptr),hl
	jp expect_eol						;if there is, then skip the THEN token and execute the block

#ifdef 0
exec_c_runp:
	di
	ld hl,temp_name-1						;calculate the size of the temp file,
	rst 20h
	ld hl,free1_mem
	ld de,(free1_mem_ptr)
	or a
	sbc hl,de
	ld de,free2_mem
	add hl,de
	ld de,(free2_mem_ptr)
	or a
	sbc hl,de
	ld de,free3_mem
	add hl,de
	ld de,(free3_mem_ptr)
	or a
	sbc hl,de
	ld de,free4_mem
	add hl,de
	ld de,(free4_mem_ptr)
	or a
	sbc hl,de
	ld de,bmp_mem
	add hl,de
	ld de,(bmp_mem_ptr)
	or a
	sbc hl,de
	ld de,spr_mem
	add hl,de
	ld de,(spr_mem_ptr)
	or a
	sbc hl,de
	ld de,func_stack
	add hl,de
	ld de,(func_sp)
	or a
	sbc hl,de
	ld de,screen_len*2+14
	add hl,de
	call _createstrng						;then create it,
	ld a,b
	ex de,hl
	call _ahl_plus_2
	call _set_abs_dest_addr
	ld hl,(free1_mem_ptr)					;and copy the data to it
	ld de,free1_mem
	xor a
	sbc hl,de
	ld (temp_buf),hl
	push hl
	ld hl,temp_buf
	call _set_abs_src_addr
	ld hl,2
	call _mm_ldir_set_size					;(first the size of first free mem block)
	pop hl
	ld a,h
	or l
	jr z,exec_c_runp_nosave1
	xor a
	call _set_mm_num_bytes
	ld hl,free1_mem
	call _set_abs_src_addr
	call _mm_ldir							;(then the contents of the memory itself)
exec_c_runp_nosave1:
	ld hl,(free2_mem_ptr)					;(now repeat the same for the others)
	ld de,free2_mem
	xor a
	sbc hl,de
	ld (temp_buf),hl
	push hl
	ld hl,temp_buf
	call _set_abs_src_addr
	ld hl,2
	call _mm_ldir_set_size
	pop hl
	ld a,h
	or l
	jr z,exec_c_runp_nosave2
	xor a
	call _set_mm_num_bytes
	ld hl,free2_mem
	call _set_abs_src_addr
	call _mm_ldir
exec_c_runp_nosave2:
	ld hl,(free3_mem_ptr)
	ld de,free3_mem
	xor a
	sbc hl,de
	ld (temp_buf),hl
	push hl
	ld hl,temp_buf
	call _set_abs_src_addr
	ld hl,2
	call _mm_ldir_set_size
	pop hl
	ld a,h
	or l
	jr z,exec_c_runp_nosave3
	xor a
	call _set_mm_num_bytes
	ld hl,free3_mem
	call _set_abs_src_addr
	call _mm_ldir
exec_c_runp_nosave3:
	ld hl,(free4_mem_ptr)
	ld de,free4_mem
	xor a
	sbc hl,de
	ld (temp_buf),hl
	push hl
	ld hl,temp_buf
	call _set_abs_src_addr
	ld hl,2
	call _mm_ldir_set_size
	pop hl
	ld a,h
	or l
	jr z,exec_c_runp_nosave4
	xor a
	call _set_mm_num_bytes
	inc a
	ld hl,free4_mem-$8000
	call _set_abs_src_addr
	call _mm_ldir
exec_c_runp_nosave4:
	ld hl,(bmp_mem_ptr)						;copy any existing bitmaps
	ld de,bmp_mem
	xor a
	sbc hl,de
	ld (temp_buf),hl
	push hl
	ld hl,temp_buf
	call _set_abs_src_addr
	ld hl,2
	call _mm_ldir_set_size
	pop hl
	ld a,h
	or l
	jr z,exec_c_runp_nobmp
	xor a
	call _set_mm_num_bytes
	inc a
	ld hl,bmp_mem-$8000
	call _set_abs_src_addr
	call _mm_ldir
exec_c_runp_nobmp:
	ld hl,(spr_mem_ptr)						;and sprites
	ld de,spr_mem
	xor a
	sbc hl,de
	ld (temp_buf),hl
	push hl
	ld hl,temp_buf
	call _set_abs_src_addr
	ld hl,2
	call _mm_ldir_set_size
	pop hl
	ld a,h
	or l
	jr z,exec_c_runp_nospr
	xor a
	call _set_mm_num_bytes
	ld hl,spr_mem
	call _set_abs_src_addr
	call _mm_ldir
exec_c_runp_nospr:
	ld hl,(func_sp)							;and the function stack
	ld de,func_stack
	xor a
	sbc hl,de
	ld (temp_buf),hl
	push hl
	ld hl,temp_buf
	call _set_abs_src_addr
	ld hl,2
	call _mm_ldir_set_size
	pop hl
	ld a,h
	or l
	jr z,exec_c_runp_nostack
	xor a
	call _set_mm_num_bytes
	ld hl,func_stack
	call _set_abs_src_addr
	call _mm_ldir
exec_c_runp_nostack:
	ld a,1									;then finally the screen buffers
	ld hl,gray1_buf-$8000
	call _set_abs_src_addr
	dec a
	ld hl,screen_len*2
	call _mm_ldir_set_size
	call expect_str							;now get the name of the program,
	ld de,(parse_ptr)						;(and save the parse address of the next line)
	ld (prog_addr_save),de
	call strlen								;make sure it's not too long,
	cp 8
	jp nc,err_progname
	ld c,a
	ld b,0
	ld d,h
	ld e,l
	inc de
	inc de
	push af
	lddr									;then copy it 2 bytes forwards to leave space for the type byte and length byte
	pop af
	ld hl,str_temp_mem+1
	ld (hl),a								;and set the length byte
	dec hl
	di
	call tok_part_1							;go through the normal tokenization process,
	ld hl,copy_addr
	ld a,(hl)
	dec a
	call z,exec_tok_prog
	call tok_part_2
	call clr_free_mem						;and run it the same as always
	ld hl,temp_name-1						;WAIT! we've got to restore the screen first!
	rst 20h
	rst 10h
	ex de,hl
	ld a,b
	call _get_word_ahl						;find the end of the file
	dec de
	add hl,de
	adc a,0
	call _set_abs_src_addr					;and copy the screen data to the buffers
	ld hl,gray2_buf-$8000+screen_len-1
	ld a,1
	call _set_abs_dest_addr
	ld hl,screen_len*2
	dec a
	call _set_mm_num_bytes
	call _mm_lddr
	ld hl,str_ti86
	ld de,free1_mem
	ld bc,str_ti86_len
	ldir
	ld hl,func_stack
	ld (func_sp),hl
	ei
	call exec
	di
	ld hl,temp_name-1						;WAIT! the screen has to be saved before the program is restored!
	rst 20h
	rst 10h
	ex de,hl
	ld a,b
	call _get_word_ahl
	dec de
	add hl,de
	adc a,0
	call _set_abs_dest_addr
	ld hl,gray2_buf-$8000+screen_len-1
	ld a,1
	call _set_abs_src_addr
	ld hl,screen_len*2
	dec a
	call _set_mm_num_bytes
	call _mm_lddr
	di
	ld hl,prog_name_save-1					;now re-tokenize the original program
	call tok_part_1
	ld hl,copy_addr
	ld a,(hl)
	dec a
	call z,exec_tok_prog
	call tok_part_2
	ld hl,temp_name-1						;when that's done, restore all the vars/bmps/sprites/etc.
	rst 20h
	rst 10h
	ex de,hl
	ld a,b
	call _ahl_plus_2
	call _set_abs_src_addr

	xor a
	ld hl,temp_buf
	call _set_abs_dest_addr
	ld hl,2
	call _mm_ldir_set_size
	ld hl,(temp_buf)
	push hl
	ld a,h
	or l
	jr z,exec_c_runp_norstr1
	xor a
	call _set_mm_num_bytes
	ld hl,free1_mem
	call _set_abs_dest_addr
	call _mm_ldir
exec_c_runp_norstr1:
	pop hl
	ld de,free1_mem
	add hl,de
	ld (free1_mem_ptr),hl
	xor a
	ld hl,temp_buf
	call _set_abs_dest_addr
	ld hl,2
	call _mm_ldir_set_size
	ld hl,(temp_buf)
	push hl
	ld a,h
	or l
	jr z,exec_c_runp_norstr2
	xor a
	call _set_mm_num_bytes
	ld hl,free2_mem
	call _set_abs_dest_addr
	call _mm_ldir
exec_c_runp_norstr2:
	pop hl
	ld de,free2_mem
	add hl,de
	ld (free2_mem_ptr),hl
	xor a
	ld hl,temp_buf
	call _set_abs_dest_addr
	ld hl,2
	call _mm_ldir_set_size
	ld hl,(temp_buf)
	push hl
	ld a,h
	or l
	jr z,exec_c_runp_norstr1
	xor a
	call _set_mm_num_bytes
	ld hl,free3_mem
	call _set_abs_dest_addr
	call _mm_ldir
exec_c_runp_norstr3:
	pop hl
	ld de,free3_mem
	add hl,de
	ld (free3_mem_ptr),hl
	xor a
	ld hl,temp_buf
	call _set_abs_dest_addr
	ld hl,2
	call _mm_ldir_set_size
	ld hl,(temp_buf)
	push hl
	ld a,h
	or l
	jr z,exec_c_runp_norstr1
	xor a
	call _set_mm_num_bytes
	inc a
	ld hl,free4_mem-$8000
	call _set_abs_dest_addr
	call _mm_ldir
exec_c_runp_norstr4:
	pop hl
	ld de,free4_mem
	add hl,de
	ld (free4_mem_ptr),hl
	xor a
	ld hl,temp_buf
	call _set_abs_dest_addr
	ld hl,2
	call _mm_ldir_set_size
	ld hl,(temp_buf)
	push hl
	ld a,h
	or l
	jr z,exec_c_runp_norstrbmp
	xor a
	call _set_mm_num_bytes
	inc a
	ld hl,bmp_mem-$8000
	call _set_abs_dest_addr
	call _mm_ldir
exec_c_runp_norstrbmp:
	pop hl
	ld de,bmp_mem
	add hl,de
	ld (bmp_mem_ptr),hl
	xor a
	ld hl,temp_buf
	call _set_abs_dest_addr
	ld hl,2
	call _mm_ldir_set_size
	ld hl,(temp_buf)
	push hl
	ld a,h
	or l
	jr z,exec_c_runp_norstrspr
	xor a
	call _set_mm_num_bytes
	ld hl,spr_mem
	call _set_abs_dest_addr
	call _mm_ldir
exec_c_runp_norstrspr:
	pop hl
	ld de,spr_mem
	add hl,de
	ld (spr_mem_ptr),hl
	xor a
	ld hl,temp_buf
	call _set_abs_dest_addr
	ld hl,2
	call _mm_ldir_set_size
	ld hl,(temp_buf)
	push hl
	ld a,h
	or l
	jr z,exec_c_runp_norstrstack
	xor a
	call _set_mm_num_bytes
	ld hl,func_stack
	call _set_abs_dest_addr
	call _mm_ldir
exec_c_runp_norstrstack:
	pop hl
	ld de,func_stack
	add hl,de
	ld (func_sp),hl

	ld hl,gray1_buf-$8000					;now restore the screen buffers
	ld a,1
	call _set_abs_dest_addr
	dec a
	ld hl,screen_len*2
	call _mm_ldir_set_size
	ld hl,temp_name-1						;and delete the temp var (finally!!!)
	rst 20h
	rst 10h
	call _delvar
	call _ram_page_1						;then go back to page 1,
	call clr_scrn							;clear the physical screen,
	call copy_scrn							;and copy the screen buffers back
	ei
	ld hl,(prog_addr_save)					;restore the address of the next line to parse
	ld (parse_ptr),hl
	ld hl,eof								;and set the write pointer to point to an EOF token,
	ld (write_ptr),hl						;so if an error occurs, the error handler can tell the program is in the execution stage
	ret
#endif

;//////////////////////////////Argument retrieval/processing routines//////////////////////////////

expect_eol:							;if not end of line, show error
	ld hl,(parse_ptr)
	ld a,(hl)							;look at the next token
	cp eol_token
	jp nz,err_eol						;if it isn't the end of the line, show error
	inc hl								;otherwise, move pointer to next line
	ld (parse_ptr),hl
	ret


expect_num:							;get number or var in hl, show error if not number
	call get_expr_val
	ld hl,(expr_sum)
	ret


expect_str:							;get pointer to string in hl, show error if not string
	ld hl,(parse_ptr)
	jp get_str


expect_intname:						;get var address in de, show error if not var, if array element, index returned in expr_sum
	call read_varname
	ld (curr_var_type),a
	cp int_name_token					;return if it's just the name of an integer
	ret z
	cp ptr_name_token
	jr z,expect_intname_ptr
	cp array_name_token
	jp nz,err_var_int
	ld a,(hl)
	cp index_start_token
	jp nz,err_var_int
	push de
	inc hl
	call get_expr_val_hl				;otherwise, get the array index
	ld hl,(expr_sum)
	ld (curr_var_index),hl				;and store it
	ld hl,(parse_ptr)
	inc hl
	ld (parse_ptr),hl
	pop de
	ret
expect_intname_ptr:						;if it's not a varname, it's also allowed to be a pointer
	call get_ptr						;so get the pointer type
	or a
	jp z,err_noptr
	cp int_char
	jp nz,err_var_int
	ld (curr_var_type),a
	ex de,hl							;then get the var's address in de
	ret									;(right where it should be) and return


expect_arrayname:					;get var address in de, show error if not array
	call read_varname
	cp array_name_token
	ret z
expect_arrayname_ptr:
	cp ptr_name_token
	jp nz,err_var_arry
	call get_ptr
	or a
	jp z,err_ptr
	cp array_char
	jp nz,err_var_arry
	ex de,hl
	ret


expect_strname:						;get var address in de, show error if not string
	call read_varname
	cp str_name_token
	ret z
expect_strname_ptr:
	cp ptr_name_token
	jp nz,err_var_str
	call get_ptr
	or a
	jp z,err_noptr
	cp string_char
	jp nz,err_var_str
	ex de,hl
	ret


expect_varname:						;get var address in de, var type (char) in a
	call read_varname
	ld (curr_var_type),a
	call tok_to_char
	ld a,b
	cp pointer_char
	ret nz
expect_varname_ptr:
	call get_ptr
	or a
	jp z,err_noptr
	ex de,hl
	ret


expect_sprname:						;get sprite address in de, show error if not sprite
	call read_varname
	cp spr_name_token
	ret z
expect_sprname_ptr:
	cp ptr_name_token
	jp nz,err_spr
	call get_ptr
	or a
	jp z,err_noptr
	cp sprite_char
	jp nz,err_spr
	ex de,hl
	ret


expect_bmpname:
	call get_bmp
	ld d,b
	ld e,c
	ret
;	call read_varname
;	cp bmp_name_token
;	ret z
;expect_bmpname_ptr:
;	cp ptr_name_token
;	jp nz,err_bmp
;	call get_ptr
;	ex de,hl
;	or a
;	jp z,err_noptr
;	cp bitmap_char
;	ret z
;	cp sprite_char
;	jp nz,err_bmp
;	ld hl,(parse_ptr)
;	ld a,(hl)
;	cp LcapB
;	jp nz,err_bmp
;	inc hl
;	ld (parse_ptr),hl
;	jp get_spr


expect_ptrname:
	call read_varname
	cp ptr_name_token
	ret z
	jp err_ptr


expect_mapname:
	call read_varname
	cp map_name_token
	ret z
expect_mapname_ptr:
	cp ptr_name_token
	jp nz,err_map
	call get_ptr
	or a
	jp z,err_noptr
	cp map_char
	jp nz,err_map
	ex de,hl
	ret


#ifdef FAST_KEY
expect_keycode:
	call read_varname
	cp keycode_token
	ret z
	jp err_num_key
#endif


read_varname:
	ld hl,(parse_ptr)
	ld a,(hl)							;get first the type token
	inc hl
	ld e,(hl)							;then the address
	inc hl
	ld d,(hl)
	inc hl
	ld (parse_ptr),hl					;and update the parse pointer
	ret


expect_x_pos:
	call expect_num
	ld a,h								;make sure that it isn't out of bounds
	or a
	jp nz,find_eol
	ld a,l
	cp 96
	ret c
	jp find_eol


expect_y_pos:
	call expect_num
	ld a,h
	or a
	jp nz,find_eol
	ld a,l
	cp 64
	ret c
	jp find_eol


expect_color:
	call expect_num
format_color:
	ld a,h								;yet again, make sure it isn't out of range...
	or a
	jp nz,err_argrange
	ld a,l
	cp num_colors
	jp nc,err_argrange
	ret


expect_color_draw:						;read color, draw w/handler in hl, no draw if transparent
	push hl
	call expect_color					;get the color
	pop hl
	or a								;and branch depending on it
	jr z,expect_color_draw_white
	dec a
	jr z,expect_color_draw_lgray
	dec a
	jr z,expect_color_draw_dgray
	dec a
	jr z,expect_color_draw_black
	dec a
	jr z,expect_color_draw_inv
	ret
expect_color_draw_inv:
	ld bc,pixel_inv
	ld (pixel_op_patch),bc
	ld a,gray1_buf/256
	ld de,$+5
	push de
	jp (hl)
	ld a,gray2_buf/256
	jp (hl)
expect_color_draw_white:
	ld bc,pixel_off
	ld (pixel_op_patch),bc
	ld a,gray1_buf/256
	ld de,$+5
	push de
	jp (hl)
	ld a,gray2_buf/256
	jp (hl)
expect_color_draw_lgray:
	ld bc,pixel_on
	ld (pixel_op_patch),bc
	ld a,gray1_buf/256
	ld de,$+5
	push de
	jp (hl)
	ld bc,pixel_off
	ld (pixel_op_patch),bc
	ld a,gray2_buf/256
	jp (hl)
expect_color_draw_dgray:
	ld bc,pixel_off
	ld (pixel_op_patch),bc
	ld a,gray1_buf/256
	ld de,$+5
	push de
	jp (hl)
	ld bc,pixel_on
	ld (pixel_op_patch),bc
	ld a,gray2_buf/256
	jp (hl)
expect_color_draw_black:
	ld bc,pixel_on
	ld (pixel_op_patch),bc
	ld a,gray1_buf/256
	ld de,$+5
	push de
	jp (hl)
	ld a,gray2_buf/256
	jp (hl)

;//////////////////////////////Other commonly used routines//////////////////////////////

exec_ret:
	ret

exec_halt:								;end prog
	pop hl
	pop hl								;modify stack to return from executor
	ret


find_eol:								;skip the current line being executed
	ld hl,(parse_ptr)
find_eol_hl:
	call goto_eol
	inc hl
	ld (parse_ptr),hl
	ld sp,(exec_sp)
	jp exec_loop


read_var_addr:							;read var address from (hl), store in curr_var
	inc hl
	ld e,(hl)
	inc hl
	ld d,(hl)
	ld (curr_var),de
	inc hl
	ret


goto_eol:								;start of current line is hl, return end of line in hl
	ld a,(hl)
	cp eol_token
	ret z
	cp eof_token
;	jp z,err_syntax
	ret z
	cp frun_token
	jp z,goto_eol_3
	cp lit_str_token
	jr z,goto_eol_str
	cp lit_num_token
	jr c,goto_eol_1
goto_eol_3:
	inc hl
	inc hl
goto_eol_1:
	inc hl
	jp goto_eol
goto_eol_str:
	ld bc,str_max_len+1
	xor a
	cpir
	jr goto_eol


parse_find_array_element:				;parse array index w/array in de, return element's address in hl
	ld hl,(parse_ptr)
parse_find_array_element_hl:
	push de								;save the location of the array's data
	ld a,(hl)							;make sure it's an array element, not a full array
	cp index_start_token
	jp nz,err_var_int					;no no no...bad user!!!
	inc hl
	call get_expr_val_hl				;now get the index of the array element
	inc hl
	ld (parse_ptr),hl					;and update the parse pointer
	pop de
	ld hl,(expr_sum)					;now find the location of the correct element in the array
	ld a,h								;(make sure it's not out of bounds)
	or a
	jp nz,err_arrayindex
	ld a,(de)
	cp l
	jp c,err_arrayindex
	inc de
	add hl,hl
	add hl,de
	ret


parse_find_map_element:					;parse map index w/map in de, return address in hl
	ld hl,(parse_ptr)
parse_find_map_element_hl:
	push de
	call get_expr_val_hl				;get the x pos
	ld hl,(expr_sum)
	push hl
	call get_expr_val					;then the y pos
	inc hl
	ld (parse_ptr),hl
	ld bc,(expr_sum)
	pop hl
	pop de
;	jr find_map_element					;using those, find the address of the map element

find_map_element:						;find map element with x position in hl, y position in bc, var address in de, return address in hl
	push hl
	push bc
	call get_var
	inc hl
	inc hl
	ld b,(hl)							;get the size of the map
	inc b
	inc hl
	ld c,(hl)
	inc c
	inc hl
	ld (find_map_element_addr),hl
	pop hl								;hl=y pos
	ld a,h
	or a
	jp nz,err_arrayindex
	ld a,l
	cp c
	jp nc,err_arrayindex
	pop de								;de=x pos
	ld a,d
	or a
	jp nz,err_arrayindex
	ld a,e
	cp b
	jp nc,err_arrayindex
	push de
	ld h,b
	dec h
	call _htimesl						;hl=y*width
	pop de
	add hl,de							;hl=x+(y*width)
	add hl,hl
find_map_element_addr	equ	$+1
	ld de,0
	add hl,de							;hl=address of map element
	ret


tok_to_char:							;var type token in a, char returned in b
	ld b,map_char
	sub map_name_token
	ret z
	ld b,sprite_char
	dec a
	ret z
	ld b,bitmap_char
	dec a
	ret z
	ld b,pointer_char
	dec a
	ret z
	ld b,string_char
	dec a
	ret z
	ld b,array_char
	dec a
	ret z
	ld b,int_char
	dec a
	ret z
	jp err_invarg


format_spr_pos:							;sprite position in hl, returned in a
	call is_hl_neg
	jr c,format_spr_pos_skip
	ld a,h
	cp $ff
	jp nz,err_argrange
	ld a,l
	cp 128
	jp c,err_argrange
	ld a,l
	ret
format_spr_pos_skip:
	ld a,h
	or a
	jp nz,err_argrange
	ld a,l
	ret

.end