;MLC (Multi-platform Language for Calculators) by Epic Programming Studios
;TI-86 interpreter
;Written by Donald Straney (burntfuse) - straney@umbi.umd.edu

;Graphics routines


;input: hl is start of zero-terminated string
;output: string drawn in all gfx buffers

draw_text_all:
	call draw_text_1
;	jr draw_text_2
draw_text_2:
	ld a,gray2_buf/256
	call draw_text96
	ld a,scrn2_buf/256
	jp draw_text128
draw_text_1:
	ld a,gray1_buf/256
	call draw_text96
	ld a,scrn1_buf/256
	jp draw_text128


;input: hl is start of zero-terminated string, a is MSB of buffer
;output: string drawn in buffer

draw_text96:
	ld (fvputc_vidplane),a
;	add a,screen_len/256
;	ld (fvputc_vidplane2),a
	push hl
	ld hl,(temp_buf)
	call fvputs
	pop hl
	ret
draw_text128:
	ld (fvputc_vidplane),a
;	add a,full_screen_len/256
;	ld (fvputc_vidplane2),a
	push hl
	ld de,15
	ld (fvputc_scr2),de
	ld hl,(temp_buf)
	call fvputs128
	ld de,11
	ld (fvputc_scr2),de
	pop hl
	ret


;input: vidplane in a, coords are b,c - d,e
;output: box drawn on screen

box:
	ld (box_patch),a
	ld a,b
	cp d
	call nc,box_swap_bd
	ld a,c
	cp e
	call nc,box_swap_ce
	call FindPixel				;hl=address
	push af
	ld a,h
box_patch	equ	$+1
	add a,0
	ld h,a
	ld a,d
	sub b
	jr z,box_skip				;if width of 0, don't draw
	ld (box_width_patch),a		;a=width
	ld a,e
	sub c
	jr z,box_skip				;also skip if height of 0
	ld b,a						;b=height
	pop af
	ld de,12					;de=newline addition
box_loop:
	push bc
	push hl
	push af
box_width_patch	equ	$+1
	ld b,0						;b=width
box_inner_loop:
	ld c,a
	call pixel_op
	ld a,c
	rrca
	call c,box_inner_loop_inc	;if at end of this byte, move to next
	djnz box_inner_loop
box_inner_loop_end:
	pop af
	pop hl
	pop bc
	add hl,de					;hl to next line
	djnz box_loop
	ret

box_inner_loop_inc:
	inc hl
	ret

box_skip:
	pop hl
	ret

box_swap_bd:
	ld a,b
	ld b,d
	ld d,a
	ret
box_swap_ce:
	ld a,c
	ld c,e
	ld e,a
	ret

;input: (none)
;output: screen buffers cleared

clr_buf:
	ld hl,gray1_buf
	ld de,gray1_buf+1
	ld (hl),0
	ld bc,screen_len*2-1
	ldir
	ret

;input: (none)
;output: screen cleared

clr_scrn:
	ld hl,scrn1_buf
	call clr_scrn_clear
	ld hl,scrn2_buf
clr_scrn_clear:
	ld bc,1023
	ld d,h
	ld e,l
	inc e
	ld (hl),0
	ldir
	ret

;input: (none)
;output: screen buffers copied to screen

copy_scrn:
	ld hl,gray1_buf
	ld de,scrn1_buf
	call copy_scrn_buf
	ld de,scrn2_buf
copy_scrn_buf:
	ld a,64
copy_scrn_buf_loop:
	inc de
	inc de
	ld bc,12
	ldir
	inc de
	inc de
	dec a
	jr nz,copy_scrn_buf_loop
	ret


;Much thanks to Detached Solutions for these routines!
;(slightly modified from their original forms)

BufferShiftLeft1:
	ld c,64
	ld e,12
BufferShiftLeftRowLoop1:
	ld b,e
	or a
BufferShiftLeftShiftLoop1:
	rl (hl)
	dec hl
	djnz BufferShiftLeftShiftLoop1
	ld a,l
	add a,e
	ld l,a
	ld a,h
	adc a,0
	ld h,a
	set 0,(hl)
BufferShiftLeftColor	equ	$-1
	ld a,l
	sub e
	ld l,a
	ld a,h
	sbc a,0
	ld h,a
	dec c
	jr nz,BufferShiftLeftRowLoop1
	ret


BufferShiftRight1:
	ld c,64
	ld e,12
BufferShiftRightRowLoop1:
	ld b,e
	or a
BufferShiftRightShiftLoop1:
	rr (hl)
	inc hl
	djnz BufferShiftRightShiftLoop1
	ld a,l
	sub e
	ld l,a
	ld a,h
	sbc a,0
	ld h,a
	set 7,(hl)
BufferShiftRightColor	equ	$-1
	ld a,l
	add a,e
	ld l,a
	ld a,h
	adc a,0
	ld h,a
	dec c
	jr nz,BufferShiftRightRowLoop1
	ret


BufferShiftLeft2:
	ld c,64
	ld e,11
BufferShiftLeftRowLoop2:
	ld a,l
	sub e
	ld l,a
	ld a,h
	sbc a,0
	ld h,a
	ld a,(hl)
	or a
	rla
	push af
	ld a,l
	add a,e
	ld l,a
	ld a,h
	adc a,0
	ld h,a
	pop af
	ld b,12
BufferShiftLeftShiftLoop2:
	rl (hl)
	dec hl
	djnz BufferShiftLeftShiftLoop2
	dec c
	jr nz,BufferShiftLeftRowLoop2
	ret


BufferShiftRight2:
	ld c,64
	ld e,11
BufferShiftRightRowLoop2:
	ld a,l
	add a,e
	ld l,a
	ld a,h
	adc a,0
	ld h,a
	ld a,(hl)
	or a
	rra
	push af
	ld a,l
	sub e
	ld l,a
	ld a,h
	sbc a,0
	ld h,a
	pop af
	ld b,12
BufferShiftRightShiftLoop2:
	rr (hl)
	inc hl
	djnz BufferShiftRightShiftLoop2
	dec c
	jr nz,BufferShiftRightRowLoop2
	ret


;====================================================================
; DisplayHL:  (idea from SCaBBy/Mardell)    [Assembly Coder's Zenith]
;  Display HL as a 5-digit decimal number with no leading zeros
;  out: AF, HL, BC, DE destroyed
;====================================================================
;Modified
DisplayHL:
 call FormatHL
 jp _vputs

FormatHL:
 ld c,'0'  				; save ascii value for zero
 ld de,_op1+5			; point to end of the buffer
 xor a				; zero terminate string
 ld (de),a				; set last byte to zero
FormatHLl:
 call _divhlby10		; next digit
 dec de				; next position
 add a,c				; convert to ascii
 ld (de),a				; save digit
 ld a,h				; load upper byte
 or l					; check with lower byte for zero
 jr nz,FormatHLl			; loop
 ex de,hl				; point to buffer
 ret


;===============Very Fast Gray Sprite Routine (8x8) by Donald Straney==============
;
;217 bytes/4609 cycles (worst case)
;Inputs: hl-Pointer to sprite, dark layer first, then light layer,
;		 then mask
;		 bc=x,y position to draw sprite
;
;Outputs: sprite drawn to screen, b=0, c=15
;Destroyed: af,bc,de,hl,ix
;

layer1	equ	gray1_buf
layer2	equ	gray2_buf

VeryFastGraySprite8:
VeryFastGraySprite8_testvclip1:
	ld a,c
	sub 56						;should it be clipped on the bottom of the screen?
	jr c,VeryFastGraySprite8_novclip
	ld e,a
	ld a,8
	sub e						;if so, see how much should be clipped
	ret z
	jr nc,VeryFastGraySprite8_endvclip	;and set the number of rows to draw from that
VeryFastGraySprite8_testvclip2:
	xor a						;if it's clipped on the top,
	sub c
	cp 8
	ret nc
	ld e,a
	add a,l						;offset the sprite pointer
	ld l,a
	ld a,h
	adc a,0
	ld h,a
	ld a,8
	sub e
	ld c,0						;and draw that at the top of the screen
	jr VeryFastGraySprite8_endvclip
VeryFastGraySprite8_novclip:
	ld a,8
VeryFastGraySprite8_endvclip:
	ld (VeryFastGraySprite8_rows),a
	ld a,b
	cp 88						;should it be clipped on the right?
	jr c,VeryFastGraySprite8_nohclip
	cp -8						;should it be clipped on the left?
	jr c,VeryFastGraySprite8_testhclip2
VeryFastGraySprite8_testhclip1:
	ld d,0
	ld e,$77
	ld a,b
	add a,96
	ld b,a
	ld ix,-12
	jr VeryFastGraySprite8_endhclip
VeryFastGraySprite8_testhclip2:
	cp 96
	ret nc
	ld d,$77
	ld e,0
	ld ix,0
	jr VeryFastGraySprite8_endhclip
VeryFastGraySprite8_nohclip:
	ld d,$77
	ld e,$77
	ld ix,0
VeryFastGraySprite8_endhclip:
	ld (VeryFastGraySprite8_vertadjust),ix
	ld a,d
	ld (VeryFastGraySprite8_hl1),a
	ld (VeryFastGraySprite8_hl2),a
	ld a,e
	ld (VeryFastGraySprite8_hr1),a
	ld (VeryFastGraySprite8_hr2),a
	push hl
	ld l,c					;find location on screen
	ld h,0					;hl=y
	add hl,hl				;*2
	add hl,hl				;*4
	ld e,l
	ld d,h
	add hl,hl				;*8
	add hl,de				;*12
	ld a,b					;a=x
	and %11111000
	rra						;/2
	rra						;/4
	rra						;/8
	add a,l					;add that to offset
	ld l,a					;hl=address (offset)
	ld a,h
	adc a,0
	ld h,a
VeryFastGraySprite8_vertadjust	equ	$+1
	ld de,0
	add hl,de
	ld a,b
	and %00000111			;a=shift amount
VeryFastGraySprite8_params:
	ld b,a					;find amount to patch shift code
	ld a,7
	sub b					;a=(7-shift)
	add a,a
	ld c,a
	add a,a
	add a,c					;a=(7-shift)*6
	ld (VeryFastGraySprite_shiftr),a	;jr [right=(7-shift)*6]
	ld a,b
	add a,a
	ld c,a
	add a,a
	add a,c					;a=shift*6
	ld (VeryFastGraySprite_shiftl),a	;jr [left=shift*6]
	pop ix					;restore sprite address
	push hl					;and find addresses of layers
	ld de,layer2
	add hl,de
	ex de,hl				;de=layer2
	pop hl
	ld bc,layer1
	add hl,bc				;hl=layer1
VeryFastGraySprite8_rows	equ	$+1
	ld b,8					;init row counter
VeryFastGraySprite8_loop:
	push bc					;save counter
	ld b,(ix+0)				;and get first layer in b
	ld c,(ix+8)				;second layer in c
	ld a,(ix+16)			;mask in a
	cpl
	push af					;(save them for next byte)
	push bc
	call VeryFastGraySprite_shift_r
	push af
	and (hl)				;and draw to layer 1
	or b
VeryFastGraySprite8_hl1	equ	$
	ld (hl),a
	pop af
	ex de,hl				;then layer 2
	and (hl)
	or c
VeryFastGraySprite8_hl2	equ	$
	ld (hl),a
	inc hl					;move to next byte
	inc de
	pop bc					;and restore the gfx for drawing the next byte
	pop af
	call VeryFastGraySprite_shift_l
	push af
	and (hl)				;again, draw to layer 2
	or c
VeryFastGraySprite8_hr1	equ	$
	ld (hl),a
	pop af
	ex de,hl				;and layer 1
	and (hl)
	or b
VeryFastGraySprite8_hr2	equ	$
	ld (hl),a
	ld bc,11				;now move to next line
	add hl,bc
	ex de,hl
	add hl,bc
	ex de,hl
	inc ix					;point to the next byte in sprite
	pop bc					;restore loop counter
	dec b
	jp nz,VeryFastGraySprite8_loop	;and loop!
	ret


;===============Very Fast Gray Sprite Routine (16x16) by Donald Straney==============
;
;335 bytes/16443 cycles (worst case)
;Inputs: hl-Pointer to sprite, dark layer first, then light layer,
;		 then mask
;		 bc=x,y position to draw sprite
;
;Outputs: sprite drawn to screen, b=0, c=14
;Destroyed: af,bc,de,hl,ix
;

VeryFastGraySprite16:
	ld a,c
	sub 48
	jr c,VeryFastGraySprite16_novclip
	ld e,a
	ld a,16
	sub e
	ret z
	jr nc,VeryFastGraySprite16_endvclip
VeryFastGraySprite16_testvclip2:
	xor a
	sub c
	cp 16
	ret nc
	ld e,a
	add a,a
	add a,l
	ld l,a
	ld a,h
	adc a,0
	ld h,a
	ld a,16
	sub e
	ld c,0
	jr VeryFastGraySprite16_endvclip
VeryFastGraySprite16_novclip:
	ld a,16
VeryFastGraySprite16_endvclip:
	ld (VeryFastGraySprite16_rows),a
	ld a,b
	cp 80
	jr c,VeryFastGraySprite16_nohclip
	cp 88
	jr c,VeryFastGraySprite16_testhclip4		;clip right byte only
	cp -16
	jr c,VeryFastGraySprite16_testhclip3		;clip left byte, cut right
	cp -8
	jr c,VeryFastGraySprite16_testhclip2		;clip right byte, cut left
VeryFastGraySprite16_testhclip1:				;clip left byte only
	ld a,b
	add a,96
	ld b,a
	ld ix,-12
	push bc
	ld de,$0077
	ld bc,$7777
	jr VeryFastGraySprite16_endhclip
VeryFastGraySprite16_testhclip2:
	ld a,b
	add a,96
	ld b,a
	ld ix,-12
	push bc
	ld de,$0000
	ld bc,$0077
	jr VeryFastGraySprite16_endhclip
VeryFastGraySprite16_testhclip3:
	cp 128
	ret nc
	push bc
	ld de,$7700
	ld bc,$0000
	ld ix,0
	jr VeryFastGraySprite16_endhclip
VeryFastGraySprite16_testhclip4:
	push bc
	ld de,$7777
	ld bc,$7700
	ld ix,0
	jr VeryFastGraySprite16_endhclip
VeryFastGraySprite16_nohclip:
	push bc
	ld de,$7777
	ld bc,$7777
	ld ix,0
VeryFastGraySprite16_endhclip:
	ld (VeryFastGraySprite16_vertadjust),ix
	ld a,d					;d=left byte left section
	ld (VeryFastGraySprite16_hl1),a
	ld (VeryFastGraySprite16_hl2),a
	ld a,b					;b=left byte right section
	ld (VeryFastGraySprite16_hl3),a
	ld (VeryFastGraySprite16_hl4),a
	ld a,e					;e=right byte left section
	ld (VeryFastGraySprite16_hr1),a
	ld (VeryFastGraySprite16_hr2),a
	ld a,c					;c=right byte right section
	ld (VeryFastGraySprite16_hr3),a
	ld (VeryFastGraySprite16_hr4),a
	pop bc
	push hl
	ld l,c					;find location on screen
	ld h,0					;hl=y
	add hl,hl				;*2
	add hl,hl				;*4
	ld e,l
	ld d,h
	add hl,hl				;*8
	add hl,de				;*12
	ld a,b					;a=x
	and %11111000
	rra						;/2
	rra						;/4
	rra						;/8
	add a,l					;add that to offset
	ld l,a					;hl=address (offset)
	ld a,h
	adc a,0
	ld h,a
VeryFastGraySprite16_vertadjust	equ	$+1
	ld de,0
	add hl,de
	ld a,b
	and %00000111			;a=shift amount
VeryFastGraySprite16_params:
	ld b,a					;find amount to patch shift code
	ld a,7
	sub b					;a=(7-shift)
	add a,a
	ld c,a
	add a,a
	add a,c					;a=(7-shift)*6
	ld (VeryFastGraySprite_shiftr),a	;jr [right=(7-shift)*6]
	ld a,b
	add a,a
	ld c,a
	add a,a
	add a,c					;a=shift*6
	ld (VeryFastGraySprite_shiftl),a	;jr [left=shift*6]
	pop ix					;restore sprite address
	push hl					;and find addresses of layers
	ld de,layer2
	add hl,de
	ex de,hl				;de=layer2
	pop hl
	ld bc,layer1
	add hl,bc				;hl=layer1
VeryFastGraySprite16_rows	equ	$+1
	ld b,16					;init row counter
VeryFastGraySprite16_loop:
	push bc					;save counter
	ld b,(ix+0)				;and get first layer in b
	ld c,(ix+32)			;second layer in c
	ld a,(ix+64)			;mask in a
	cpl
	push af					;(save them for next byte)
	push bc
	call VeryFastGraySprite_shift_r	;and shift all of them!
	push af
	and (hl)				;and draw to layer 1
	or b
VeryFastGraySprite16_hl1	equ	$
	ld (hl),a
	pop af
	ex de,hl				;then layer 2
	and (hl)
	or c
VeryFastGraySprite16_hl2	equ	$
	ld (hl),a
	inc hl					;move to next byte
	inc de
	pop bc					;and restore the gfx for drawing the next byte
	pop af
	call VeryFastGraySprite_shift_l	;shift them all left this time
	push af
	and (hl)				;again, draw to layer 2
	or c
VeryFastGraySprite16_hr1	equ	$
	ld (hl),a
	pop af
	ex de,hl				;and layer 1
	and (hl)
	or b
VeryFastGraySprite16_hr2	equ	$
	ld (hl),a
	inc ix					;point to the next byte in sprite
	ld b,(ix+0)				;and repeat the process for the right side!
	ld c,(ix+32)
	ld a,(ix+64)
	cpl
	push af
	push bc
	call VeryFastGraySprite_shift_r
	push af
	and (hl)
	or b
VeryFastGraySprite16_hl3	equ	$
	ld (hl),a
	pop af
	ex de,hl
	and (hl)
	or c
VeryFastGraySprite16_hl4	equ	$
	ld (hl),a
	inc hl
	inc de
	pop bc
	pop af
	call VeryFastGraySprite_shift_l
	push af
	and (hl)
	or c
VeryFastGraySprite16_hr3	equ	$
	ld (hl),a
	pop af
	ex de,hl
	and (hl)
	or b
VeryFastGraySprite16_hr4	equ	$
	ld (hl),a
	inc ix
	ld bc,10				;move to next line
	add hl,bc
	ex de,hl
	add hl,bc
	ex de,hl
	pop bc					;restore loop counter
	dec b
	jp nz,VeryFastGraySprite16_loop	;and loop!
	ret


VeryFastGraySprite_shift_r:
VeryFastGraySprite_shiftr	equ	$+1
	jr $
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	srl a
	srl b
	srl c
	cpl
	ret

VeryFastGraySprite_shift_l:
VeryFastGraySprite_shiftl	equ	$+1
	jr $
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	sla a
	sla b
	sla c
	cpl
	ret


;
; FASTLINE 1.0
;
; BRESENHAM'S LINE ALGORITHM
; by Jimmy Mrdell <yarin@acc.umu.se>
;
; Parameters:  BC    = memory area to draw line (usually $FC00)
;              DE,HL = coordinates of line (D,E)-(H,L)
;                      0 < D,H < 128, 0 < E,L < 64
;
; Code size:   145 bytes (excluding FindPixel)
;             
; Registers:   AF destroyed.
;
; Performance: 4 times faster than Stephane Jantzens line routine
;
;              Delta_X > Delta_Y  =>  innerloop =  98.50 T states
;              Delta_Y > Delta_X  =>  innerloop = 106.75 T states
;

FLine:
 push bc
; push de
; push hl

#ifdef FAST_LINES
;Modification
 ld a,h
 cp d
 jp z,FLine_vert
 ld a,l
 cp e
 jp z,FLine_horiz
#endif
 call cphlde
 jp z,FLine_pixel

 push bc
 ld a,h
 cp d
 jr nc,DXpos
 ex de,hl
DXpos:
 ld b,d
 ld c,e
 pop de
 push hl
 call FindPixel
 add hl,de
 ex (sp),hl
 ld e,c
 ld c,a
 ld a,h
 sub b
 ld b,a
 ld a,l
 sub e
 ld de,12
 jr nc,DYpos
 neg
 ld de,-12
DYpos:
 push af
 sub b
 jr nc,DY_Greater
 add a,a
 neg
 ld (x_ai+1),a
 pop af
 add a,a
 ld (XDNeg+1),a
 sub b
 pop hl
 push af
 ld a,c
;Modification
 call pixel_op
; or (hl)
; ld (hl),a
 pop af
 bit 7,a
 jr nz,XDNeg
XNewRow:
 add hl,de
x_ai:
 sub 0
 push af
 rrc c
 jr nc,XNoWrap1
 inc hl
XNoWrap1:
 ld a,c
;Modification
 call pixel_op
; or (hl)
; ld (hl),a
 pop af
 dec b
 jr z,LineDone
 jr nc,XNewRow
XDNeg:
 add a,0
 push af
 rrc c
 jr nc,XNoWrap2
 inc hl
XNoWrap2:
 ld a,c
;Modification
 call pixel_op
; or (hl)
; ld (hl),a
 pop af
 dec b
 jr z,LineDone
 jr c,XNewRow
 jr XDNeg
DY_Greater:
 neg
 add a,a
 ld (YNoWrap+1),a
 ld a,b
 add a,a
 ld (YDNeg+1),a
 pop hl
 sub h
 ld b,h
 pop hl
 push af
 ld a,c
;Modification
 call pixel_op
; or (hl)
; ld (hl),a
 pop af
YRepeat:
 bit 7,a
 jr nz,YDNeg
 rrc c
 jr nc,YNoWrap
 inc hl
YNoWrap:
 add a,0
 jr YPlot
YDNeg:
 add a,0
YPlot:
 add hl,de
 push af
 ld a,c
;Modification
 call pixel_op
; or (hl)
; ld (hl),a
 pop af
 djnz YRepeat
LineDone:
; pop hl
; pop de
 pop bc
 ret

;More modifications
pixel_op:
pixel_op_patch	equ	$+1
 jp 0
pixel_on:
 or (hl)
 ld (hl),a
 ret
pixel_off:
 cpl
 and (hl)
 ld (hl),a
 ret
pixel_inv:
 xor (hl)
 ld (hl),a
 ret


FLine_pixel:
 push bc
 ld b,h
 ld c,l
 call FindPixel
 pop de
 add hl,de
 call pixel_op
; pop hl
; pop de
 pop bc
 ret


#ifdef FAST_LINES

FLine_vert:
 ld a,l
 cp e
 jr c,FLine_vert_skip
 ex de,hl
FLine_vert_skip:		;hl->de, down
 ld a,e
 sub l
 inc a
 push af
 push bc
 ld b,h
 ld c,l
 call FindPixel
 pop bc
 add hl,bc
 pop bc					;b=height
 ld de,12
FLine_vert_loop:
 ld c,a
 call pixel_op
 ld a,c
 add hl,de
 djnz FLine_vert_loop
; pop hl
; pop de
 pop bc
 ret

FLine_horiz:
 ld a,h
 cp d
 jr c,FLine_horiz_skip
 ex de,hl
FLine_horiz_skip:		;hl->de, right
 ld a,d
 sub h
 inc a
 push af
 push bc
 ld b,h
 ld c,l
 call FindPixel
 pop bc
 add hl,bc
 pop bc					;b=width
FLine_horiz_loop:
 ld c,a
 call pixel_op
 ld a,c
 rrca
 call c,FLine_horiz_inc
 djnz FLine_horiz_loop
; pop hl
; pop de
 pop bc
 ret
FLine_horiz_inc:
 inc hl
 ret

#endif

FindPixel:	;bc is pos?
 push bc
 push de
 ld hl,ExpTable
 ld d,0
 ld a,b
 and $07
 ld e,a
 add hl,de
 ld e,(hl)
 ld h,d
 srl b
 srl b
 srl b
 ld a,c
 add a,a
 add a,a
 ld l,a
 push bc
 ld c,l
 ld b,h
 add hl,hl
 add hl,bc
 pop bc
 ld a,e
 ld e,b
 add hl,de
 pop de
 pop bc
 ret

ExpTable:
 .db $80,$40,$20,$10,$08,$04,$02,$01

.end