;MLC (Multi-platform Language for Calculators) by Epic Programming Studios
;TI-86 interpreter
;Written by Donald Straney (burntfuse) - straney@umbi.umd.edu

;Interrupt code

;//////////////////////////////The ISR//////////////////////////////

;Taken from grayscale in Diamonds, which was taken from Bomberbloke by Matthew Shepcar :-)

int_start:
	exx
	ex af,af'
;Modification
	in a,(3)						;if ON has been pressed,
	bit 3,a
;	jr z,int_exit					;goto the exit routine
	call z,open_option_menu

	in a,(3)
	bit 3,a
	jr z,no_down_left_bug
	and %11111110
	out (3),a
no_down_left_bug:
	bit 1,a
	jr z,end_int
	ld hl,gray_cntr
	inc (hl)
	ld a,(hl)
	cp 3
	ld a,$3c	;$3c
	jr c,set_video_port
	ld a,$0a	;$0a
	ld (hl),0
set_video_port:
	out (0),a
end_int:
;Next section from ISR template by Matt Johnson
    in a,(3)                            ; Get byte from Port 3
    rra                                 ; Check Bit 0 (ON interrupt status) by using carry
    ld  a,9
    adc a,0                             ; A = $09 if ON interrupt did not occur, $0A if (carry) so
    out (3),a                           ; Write either $09 or $0A to Port 3
    ld a,$0B                                                        
    out (3),a                           ; Write $0B to Port 3
    ex af,af'
    exx
    ei
    reti

;int_exit:
;	ld sp,(sp_save)						;reset the sp,
;	ld a,$0d							;make sure that the right ROM page is loaded,
;	out (5),a
;	jp end_prog							;and go back to the shell

int_end:

gray_cntr: .db 0

;//////////////////////////////Installation//////////////////////////////

open_gray:
	ld hl,vector_table_addr
	ld de,vector_table_addr+1
	ld (hl),isr_addr/256
	ld bc,257
	ldir
	ld hl,int_start
	ld de,isr_addr
	ld bc,int_end-int_start
	ldir
	ld a,vector_table_addr/256
	ld i,a
	im 2
	ei
	ret

close_gray:
    im 1
    ld a,$3C
    out (0),a
	ret

;//////////////////////////////In-game option menu//////////////////////////////

open_option_menu:
	in a,(3)							;first wait until the on key isn't pressed anymore
	bit 3,a
	jr z,open_option_menu
	pop de
	ld bc,option_menu					;set to return to the option menu when finished with this interrupt
	push bc
	push de
	ret

option_menu:
	push af
	push bc
	push de
	push hl
	in a,(5)							;save the current ROM page,
	push af
	ld a,$0d							;then make sure that the right one is loaded,
	out (5),a
	ld a,(iy+textflags)					;save the text flags,
	push af
	set textInverse,(iy+textflags)
	call close_gray						;go back to the standard interrupt mode,
option_menu_restart:
	ld hl,$fc00							;then draw the menu box on the screen
	call fill_scrn
	ld b,option_menu_text_lines			;and the text in it
	ld de,option_menu_pos
	ld (_pencol),de
	ld a,e
	ld (_vnewline_patch),a
	ld hl,option_menu_text
option_menu_draw_text_loop:
	push bc
	call _vputs
	call _vnewline
	pop bc
	djnz option_menu_draw_text_loop
option_menu_getkey:
	call wait_for_a_key					;then wait for a keypress
	push af
	call wait_for_no_key
	pop af
	cp 11
	jr z,option_menu_ret				;f1 = return to game
	cp 12
	jr z,option_menu_exit				;f2 = exit game
	cp 13
	call z,turn_off_calc				;f3 = power off
	cp 14
	jr z,option_menu_speed				;f4 = adjust WAIT speed
	cp 15
	jr z,option_menu_contrast			;f5 = adjust contrast
	cp 22
	jr nz,option_menu_getkey			;exit key also returns to game
option_menu_ret:
	call copy_scrn						;when done, restore the screen,
	pop af
	ld (iy+textflags),a					;restore the text settings,
	pop af
	out (5),a
	ld a,$ff							;reset the keypad in case it was in the middle of a keypress detection,
	out (1),a
	pop hl								;restore the registers,
	pop de
	pop bc
	pop af
	im 2								;open the grayscale handler again,
	ret									;and let the execution code continue where it left off!
option_menu_exit:
	ld sp,(sp_save)						;reset the sp,
	jp end_prog							;and terminate the program
option_menu_speed:
	ld a,(wait_adjust)
	call draw_val_adjust
	call wait_for_a_key
	ld hl,wait_adjust
	cp 26
	jr nz,$+3
	inc (hl)
	cp 24
	jr nz,$+3
	dec (hl)
	cp 21
	jp z,option_menu_restart
	call wait_for_no_key
	call erase_val_adjust
	jr option_menu_speed
option_menu_contrast:
	ld a,(_contrast)
	call draw_val_adjust
	call wait_for_a_key
	ld hl,_contrast
	cp 26
	jr nz,option_menu_contrast_skip1
	inc (hl)
	ld a,(hl)
	out (2),a
option_menu_contrast_skip1:
	cp 24
	jr nz,option_menu_contrast_skip2
	dec (hl)
	ld a,(hl)
	out (2),a
option_menu_contrast_skip2:
	cp 21
	jp z,option_menu_restart
	call wait_for_no_key
	call erase_val_adjust
	jr option_menu_contrast


draw_val_adjust:
	push af
	ld hl,option_val_pos
	ld (_pencol),hl
	ld a,$cf
	call _vputmap
	pop af
	ld l,a
	ld h,0
	call FormatHL
	call _vputs
	ld a,$05
	call _vputmap
	ld hl,option_help_pos
	ld (_pencol),hl
	ld hl,option_menu_help_text
	jp _vputs

erase_val_adjust:
	ld hl,$fc00+(option_val_pos/256*16)
	ld c,64-(option_val_pos/256)
	jr fill_scrn_loop1

fill_scrn:
	ld c,64
fill_scrn_loop1:
	inc l
	inc l
	ld b,12
fill_scrn_loop2:
	ld (hl),$ff
	inc hl
	djnz fill_scrn_loop2
	inc l
	inc hl
	dec c
	jr nz,fill_scrn_loop1
	ret

option_menu_help_text: .db "Press 2nd when done",0

.end