;MLC (Multi-platform Language for Calculators) by Epic Programming Studios
;TI-86 interpreter
;Written by Donald Straney (burntfuse) - straney@umbi.umd.edu

;Command handlers

#include "mlc.inc"

.plugin string86

.db 2									;token for ARRY
.dw 36									;length of handler

exec_c_arry:
	call expect_arrayname				;get the index of the array var and save it
	push de
	call get_var						;(make sure it hasn't been defined already)
	ld a,h
	or l
	jp nz,err_twovar_arry
	call expect_num						;now get the size of the array
	ld a,h
	or a
	jp nz,err_arraylen
	or l
	jp z,err_arraylen
	add hl,hl							;and allocate the memory
	inc hl
	ld b,h
	ld c,l
	call alloc_mem
	ld (hl),a
	pop de
	jp set_var

.db 3
.dw 43

exec_c_size:
	call expect_varname					;get index of array or string var
	push de								;then save the address w/the size
	push af
	call expect_intname					;now get the index of the var where the size will be stored
	ld (exec_c_size_var),de
	pop af								;wait...are we trying to find the size of an array or a string???
	cp array_char
	jr z,exec_c_size_array
	cp string_char
	jp nz,err_vartype					;if it's neither of those, show an error
exec_c_size_str:
	pop hl
	call _strlength						;if it's a string, use the handy API call to find its length
	ld h,b
	ld l,c
	jr exec_c_size_end
exec_c_size_array:						;if it's an array,
	pop de
	call get_var
	ld l,(hl)							;we get the array size from the first byte in the array table!
	ld h,0								;is this easy or WHAT?!?!
exec_c_size_end:
exec_c_size_var	equ	$+1
	ld de,0
	jp set_var_warray					;store the size to the var

.db 5
.dw 46

exec_c_text:							;draw small text
	call expect_num						;first get x pos
	ld a,h
	or a
	jp nz,find_eol
	ld a,l
	cp 92
	jp nc,find_eol
	ld (_pencol),a						;and set the cursor
	call expect_num						;now get the y pos
	ld a,h
	or a
	jp nz,find_eol
	ld a,l
	cp 59
	jp nc,find_eol
	ld (_penrow),a						;and set the cursor pos
	call expect_str						;now get a pointer to the string to be displayed
	ld (temp_buf),hl					;save that
	ld hl,draw_text96
	jp expect_color_draw

.db 6
.dw 33

exec_c_line:
	call expect_x_pos					;get x1
	push af
	call expect_y_pos					;then y1
	ld e,a
	pop af
	ld d,a
	ld (exec_c_line_draw2),de			;now we have the first set of coordinates...
	call expect_x_pos					;so get x2
	push af
	call expect_y_pos					;and y2
	ld l,a
	pop af
	ld h,a								;by now, both coordinates are present and accounted for
	ld (exec_c_line_draw1),hl
	ld hl,exec_c_line_draw
	jp expect_color_draw				;so draw depending on the color

.db 7
.dw 87

exec_c_pixl:
	call expect_x_pos					;get x pos
	push af
	call expect_y_pos					;then the y pos
	ld c,a
	pop af
	ld b,a
	push bc								;now we have the coordinates, so get the color
	call expect_color
	pop bc
	or a
	jr z,exec_c_pixl_white
	dec a
	jr z,exec_c_pixl_lgray
	dec a
	jr z,exec_c_pixl_dgray
	dec a
	jr z,exec_c_pixl_black
exec_c_pixl_inv:
	call exec_c_pixl_draw1
	call pixel_inv
	call exec_c_pixl_draw2
	jp pixel_inv
exec_c_pixl_white:
	call exec_c_pixl_draw1
	call pixel_off
	call exec_c_pixl_draw2
	jp pixel_off
exec_c_pixl_lgray:
	call exec_c_pixl_draw1
	call pixel_on
	call exec_c_pixl_draw2
	jp pixel_off
exec_c_pixl_dgray:
	call exec_c_pixl_draw1
	call pixel_off
	call exec_c_pixl_draw2
	jp pixel_on
exec_c_pixl_black:
	call exec_c_pixl_draw1
	call pixel_on
	call exec_c_pixl_draw2
	jp pixel_on

.db 8
.dw 59

exec_c_pixt:
	call expect_x_pos					;get the x pos of the pixel to test
	push af
	call expect_y_pos					;then its y pos
	ld c,a
	pop af
	ld b,a
	push bc
	call expect_intname					;now get the var where the color will be stored
	ld (curr_var),de
	xor a								;init the temp color var,
	ld (expr_sum),a
	pop bc
	call FindPixel						;then find the pixel in the first screen layer
	push hl
	ld b,a
	ld de,gray1_buf
	add hl,de
	and (hl)							;and test it
	call nz,exec_c_pixt_lgray
	pop hl								;now find the same pixel in the second screen layer
	ld a,b
	ld de,gray2_buf
	add hl,de
	and (hl)
	call nz,exec_c_pixt_dgray
	ld a,(expr_sum)						;get the color we've found
	ld l,a
	ld h,0
	ld de,(curr_var)					;and store it to the var
	jp set_var_warray

.db 12
.dw 12

exec_c_stop:
	ld hl,gray1_buf
	ld de,scr_save1
	ld bc,screen_len*2
	ldir
	ret

.db 13
.dw 12

exec_c_rclp:
	ld hl,scr_save1
	ld de,gray1_buf
	ld bc,screen_len*2
	ldir
	ret

.db 9
.dw 36

exec_c_rect:
	call expect_x_pos					;get x1
	ld (temp_buf+1),a					;and save it
	call expect_y_pos					;do the same with y1,
	ld (temp_buf),a
	call expect_x_pos					;x2,
	ld (temp_buf+3),a
	call expect_y_pos					;and y2
	ld (temp_buf+2),a
	ld hl,exec_c_rect_lines
	call expect_color_draw				;now get the line color, and draw the lines
	ld hl,exec_c_rect_fill
	jp expect_color_draw				;then get the fill color and draw the inside

.db 33
.dw 74

exec_c_disp:
	ld hl,(parse_ptr)
	ld a,(hl)
	cp lit_num_token
	jr z,exec_c_disp_all
	call expect_sprname					;get the sprite to draw
	call exec_c_disp_set				;update the position from VX and VY
	call get_spr						;now get the sprite's position
	ld a,d
	or e
	jp z,err_nospr
	jp draw_bmp							;and draw the sprite
exec_c_disp_all:						;if there isn't a sprite name, it must be a .0
	inc hl								;(get the number just to make sure it is a 0)
	ld b,(hl)
	inc hl
	ld a,(hl)
	inc hl
	ld (parse_ptr),hl
	or b
	jp nz,err_invarg
	ld a,(num_sprites)					;get the number of sprites to draw
	or a
	jp z,find_eol						;if there aren't any, then what's the point of drawing them???
	ld b,a
	ld de,spr_1_data					;init the pointer to the beginning of the sprites...
exec_c_disp_all_loop:
	push de
	push bc
	call exec_c_disp_set				;move the sprite if it has a velocity
	call get_spr
	ld a,d								;then make sure it's been defined,
	or e
	jp z,err_nospr
	call draw_bmp						;and draw it
	pop bc
	pop de
	ld hl,spr_data_len
	add hl,de							;now point to the next sprite,
	ex de,hl
	djnz exec_c_disp_all_loop			;and loop
	ret									;when all the sprites have been drawn, we're done

.db 10
.dw 370

exec_c_shft:
;It is illegal to copy and re-use the following code in the handler due to federal standards
;of bad coding and "spaghetti code" regulations  ;-D
	call expect_num						;get the x amount it should be shifted
	ld (shft_x),hl
	call expect_num						;then the y shift
	ld (shft_y),hl
	call expect_num						;and whether it should loop or not
	inc hl
	call format_color
	ld (shft_color),a
exec_c_shft_x:
	or a
	jr z,exec_c_shft_x_doloopX
exec_c_shft_x_dofill:
	ld hl,(shft_x)
	ld a,h
	or l
	jr z,exec_c_shft_y_dofillX
	call is_hl_neg
	jr nc,exec_c_shft_x_dofill_neg
exec_c_shft_x_dofill_pos:
	ld de,shft_x_max+1					;check for arg out of range
	call cphlde
	jp nc,err_argrange
	ld bc,$bebe							;decode color and calculate code patches
	ld a,(shft_color)
	dec a
	bit 0,a
	jr z,$+4
	ld b,$fe
	bit 1,a
	jr z,$+4
	ld c,$fe
	ld d,l
exec_c_shft_x_dofill_pos_loop:
	ld hl,gray1_buf
	ld a,b
	ld (BufferShiftRightColor),a
	push bc
	call BufferShiftRight1
	pop bc
	ld hl,gray2_buf
	ld a,c
	ld (BufferShiftRightColor),a
	push bc
	call BufferShiftRight1
	pop bc
	dec d
	jr nz,exec_c_shft_x_dofill_pos_loop
	jr exec_c_shft_y_dofill
exec_c_shft_x_doloopX:
	jr exec_c_shft_x_doloop
exec_c_shft_x_dofill_neg:
	call neg_hl
	ld de,shft_x_max+1
	call cphlde
	jp nc,err_argrange
	ld bc,$8686
	ld a,(shft_color)
	dec a
	bit 0,a
	jr z,$+4
	ld b,$c6
	bit 1,a
	jr z,$+4
	ld c,$c6
	ld d,l
exec_c_shft_x_dofill_neg_loop:
	ld hl,gray1_buf+screen_len-1
	ld a,b
	ld (BufferShiftLeftColor),a
	push bc
	call BufferShiftLeft1
	pop bc
	ld hl,gray2_buf+screen_len-1
	ld a,c
	ld (BufferShiftLeftColor),a
	push bc
	call BufferShiftLeft1
	pop bc
	dec d
	jr nz,exec_c_shft_x_dofill_neg_loop
exec_c_shft_y_dofillX:
	jr exec_c_shft_y_dofill
exec_c_shft_x_doloop:
	ld hl,(shft_x)						;choose right or left shift loops depending on sign of number
	ld a,h
	or l
	jr z,exec_c_shft_y_doloopX
	call is_hl_neg
	jr nc,exec_c_shft_x_doloop_neg
exec_c_shft_x_doloop_pos:
	ld d,l								;load loop counter
exec_c_shft_x_doloop_pos_loop:
	ld hl,gray1_buf						;shift both layers
	call BufferShiftRight2
	ld hl,gray2_buf
	call BufferShiftRight2
	dec d
	jr nz,exec_c_shft_x_doloop_pos_loop
	jr exec_c_shft_y_doloop
exec_c_shft_x_doloop_neg:
	call neg_hl
	ld d,l
exec_c_shft_x_doloop_neg_loop:
	ld hl,gray1_buf+screen_len-1
	call BufferShiftLeft2
	ld hl,gray2_buf+screen_len-1
	call BufferShiftLeft2
	dec d
	jr nz,exec_c_shft_x_doloop_neg_loop
exec_c_shft_y_doloopX:
	jr exec_c_shft_y_doloop
exec_c_shft_y_dofill:
	ld hl,(shft_y)
	ld a,h
	or l
	ret z
	ld a,(shft_color)
	dec a
	ld de,0
	bit 0,a
	jr z,$+4
	ld d,$ff
	bit 1,a
	jr z,$+4
	ld e,$ff
	call is_hl_neg
	jr nc,exec_c_shft_y_dofill_neg
exec_c_shft_y_dofill_pos:
	push de
	ld de,shft_y_max+1
	call cphlde
	jp nc,err_argrange
	add hl,hl
	add hl,hl
	ld d,h
	ld e,l
	add hl,hl
	add hl,de
	ld (shft_size),hl
	pop de
	ld a,d
	push de
	ld hl,gray1_buf+screen_len-1
	call exec_c_shft_y_pos_fill
	pop de
	ld a,e
	ld hl,gray2_buf+screen_len-1
	jp exec_c_shft_y_pos_fill
exec_c_shft_y_dofill_neg:
	call neg_hl
	push de
	ld de,shft_y_max+1
	call cphlde
	jp nc,err_argrange
	add hl,hl
	add hl,hl
	ld d,h
	ld e,l
	add hl,hl
	add hl,de
	ld (shft_size),hl
	pop de
	ld a,d
	push de
	ld hl,gray1_buf
	call exec_c_shft_y_neg_fill
	pop de
	ld a,e
	ld hl,gray2_buf
	jp exec_c_shft_y_neg_fill
exec_c_shft_y_doloop:
	ld hl,(shft_y)
	ld a,h
	or l
	ret z
	call is_hl_neg
	jr nc,exec_c_shft_y_doloop_neg
exec_c_shft_y_doloop_pos:
	add hl,hl
	add hl,hl
	ld d,h
	ld e,l
	add hl,hl
	add hl,de
	ld (shft_size),hl
	ld de,gray1_buf+screen_len-1
	call exec_c_shft_y_pos_loop
	ld de,gray2_buf+screen_len-1
	jp exec_c_shft_y_pos_loop
exec_c_shft_y_doloop_neg:
	call neg_hl
	add hl,hl
	add hl,hl
	ld d,h
	ld e,l
	add hl,hl
	add hl,de
	ld (shft_size),hl
	ld hl,gray1_buf
	call exec_c_shft_y_neg_loop
	ld hl,gray2_buf
	jp exec_c_shft_y_neg_loop


.db 14
.dw 26

exec_c_gkey:
	call expect_keycode
	push de
	call expect_intname
	pop hl
	ld a,h
	call get_key
	and l
	jr nz,exec_c_gkey_no
exec_c_gkey_yes:
	ld hl,1
	jr exec_c_gkey_end
exec_c_gkey_no:
	ld hl,0
exec_c_gkey_end:
	jp set_var_warray


.db 15
.dw 14

exec_c_wkey:
	call expect_intname					;get the name of the var to store the keypress to
	push de
	call wait_for_a_key					;loop until a key has been pressed,
	ld l,a								;then store the keycode to the var
	ld h,0
	pop de
	jp set_var_warray

.db 16
.dw 10

exec_c_paus:
	ld a,k_g6
	call get_key						;get keypress
	cp k_enter							;if it isn't enter,
	jr nz,exec_c_paus					;then loop again
	ret

.db 19
.dw 37

exec_c_frun:
	ld de,(func_sp)						;make sure the stack isn't full
	ld hl,end_func_stack
	call cphlde
	jp z,err_frun
	ld hl,(parse_ptr)					;if it isn't, get address of next command
	push hl
	inc hl
	inc hl
	inc hl
	ex de,hl							;and push it on the stack
	ld (hl),e
	inc hl
	ld (hl),d
	inc hl
	ld (func_sp),hl
	pop hl								;now get the address of the function we're jumping to
	call ldhlind
	ld (parse_ptr),hl					;and go to it
	pop hl
	ret

.db 20
.dw 25

exec_c_fend:
;	call expect_eol						;don't need no stinkin' arguments!!!
	ld de,func_stack
	ld hl,(func_sp)
	call cphlde
	jp z,exec_halt
	dec hl
	ld d,(hl)
	dec hl
	ld e,(hl)
	ld (func_sp),hl
	ld (parse_ptr),de
	pop hl
	ret

.db 21
.dw 35

exec_c_fgob:
	ld hl,(func_sp)						;first make sure we're not already at the beginning of the stack
	ld de,func_stack
	call cphlde
	jr z,exec_c_fgob_main
	dec hl								;then get the address of the FRUN that called this function
	ld d,(hl)
	dec hl
	ld e,(hl)
	ex de,hl
	dec hl								;get to the address of this function
	dec hl
	ld d,(hl)
	dec hl
	ld e,(hl)
	ld (parse_ptr),de					;and start executing there
	pop hl
	ret
exec_c_fgob_main:
	ld hl,(start_addr)
	ld (parse_ptr),hl
	pop hl
	ret

.db 28
.dw 88

exec_c_rndm:			;rand / (65535 / (upper - lower)) + lower
	call expect_num						;get the lower limit
	push hl
	call expect_num						;then the upper limit
	inc hl								;(hardly ever hits the upper limit, so we have to increase it)
;	ld (exec_c_rndm_patch_up),hl
	push hl								;and save them
	call expect_intname					;get variable where the random number will be stored
	ld (curr_var),de
	call RandomNumber					;then get the random number - first the msb
	push af
	call RandomNumber					;then the lsb
	ld c,a
	pop af
	and $03
	ld b,a
	pop hl
	pop de								;(upper limit in hl, lower limit in de, rand in bc)
	call cphlde
	push hl
	jr nc,exec_c_rndm_inrange
;	ld (exec_c_rndm_patch_ans),de
	push de

	call neg_de
	add hl,de
	ld de,0
	jr exec_c_rndm_skip0
exec_c_rndm_inrange:
	ld ix,0
;	ld (exec_c_rndm_patch_ans),ix
	push ix

exec_c_rndm_skip0:
	or a
	sbc hl,de							;hl=upper - lower
	push de
	push bc
	ex de,hl							;de=upper-lower
	ld hl,$03ff							;hl=32767
	call div16							;hl=32767/(upper-lower)
	ex de,hl							;de=32767/(upper-lower)
	pop hl								;hl=rand
	call div16							;hl=rand/(32767/(upper-lower))
	pop de
	add hl,de							;hl=rand/(32767/(upper-lower))+lower
;exec_c_rndm_patch_ans	equ	$+1
;	ld de,0
	pop de

	add hl,de
;exec_c_rndm_patch_up	equ	$+1			;because of the loss in precision (no decimals) with large numbers, results are sometimes out of range
;	ld de,0								;so we've got to check for that
	pop de
	call cp_num							;is it over the upper limit?
	jr c,exec_c_rndm_skip				;if not, everything's fine
	ld h,d								;otherwise, set it to the lower limit
	ld l,e
exec_c_rndm_skip:
	ld de,(curr_var)					;now get the var index
	jp set_var_warray					;and write the number to the var

.db 26
.dw 18

exec_c_wait:
	call expect_num						;get the time to wait,
	ld a,(wait_adjust)
	ld e,a
	ld d,0
	call mult16							;adjust it for speed changes,
exec_c_wait_loop:						;and go in a pointless loop to waste time!
	ld a,h
	or l
	ret z
	dec hl
	jr exec_c_wait_loop

.db 35
.dw 13

exec_c_sprt:
	call expect_sprname					;first get the name of the sprite
	push de
	call expect_bmpname					;then the bitmap
	ld b,d
	ld c,e
	pop de
	jp set_spr							;set that sprite to reference the bitmap
;	jp expect_eol						;that's all for now, folks!

.db 36
.dw 18

exec_c_dbmp:
	call expect_bmpname					;get the index of the bitmap
	push de
	call expect_num						;then the x pos
	push hl
	call expect_num						;and the y pos
	ld c,l
	pop hl
	ld b,l
	pop de
	jp draw_bmp							;now draw it

.db 37
.dw 75

exec_c_clsn:
	call expect_sprname					;get the position of the first sprite
	ld (exec_c_clsn_nosprite),de
	call get_spr
	ld a,d
	or e
	jp z,err_nospr
	ld (exec_c_clsn_sprite1),de
	push bc
	call get_bmp_size
	ld (exec_c_clsn_spr1size),a
	ld hl,(parse_ptr)					;(if there's a .0 instead of the second sprite, then test for collision with all others)
	ld a,(hl)
	cp spr_name_token
	jp nz,exec_c_clsn_all
	call expect_sprname					;along with the second one
	call get_spr
	ld a,d
	or e
	jp z,err_nospr
	ld (exec_c_clsn_sprite2),de
	push bc
	call get_bmp_size
	ld (exec_c_clsn_spr2size),a
	call expect_intname					;and the var to store the result in
	ld (curr_var),de
	pop hl
	pop de
	call exec_c_clsn_test				;test if there's a collision
	jp nc,exec_c_clsn_no
	jp exec_c_clsn_yes

.db 43
.dw 185

exec_c_wrte:
	call expect_num						;get position in file to write
	ld a,h								;(make sure that it's in range)
	or a
	jp nz,err_argrange
	ld (file_pos),hl
	call expect_varname					;and the data to write to it
	ld (curr_var_type),a
	cp string_char						;check what type of data it is
	jr z,exec_c_wrte_str
	call get_var
	cp array_char
	jr z,exec_c_wrte_array
exec_c_wrte_int:
	ld bc,2								;we know how long an int is!
	ld (wrte_temp),hl
	ld hl,wrte_temp
	jr exec_c_wrte_go
exec_c_wrte_str:
	ex de,hl
	call _strlength						;if it's a string, get the length of the string
	jr exec_c_wrte_go
exec_c_wrte_array:
	ld a,(hl)							;get the length of the array from the size byte
	cp array_write_len+1				;(make sure it's not too big to write)
	jp nc,err_mem_file
	rla
	inc a
	ld c,a
	ld b,0
exec_c_wrte_go:		;hl=data start, bc=data len
	push hl
	push bc
	ld hl,curr_name
	rst 20h									;get the size of the string var
	rst 10h
	call _data_size_to_de					;de=size of string
	push de
	ld hl,(file_pos)
	inc l
	ld de,data_seg_len
	call mult16
	pop de									;hl=required size of string, de=size of string
	call cphlde
	jr c,exec_c_wrte_sizefine				;if the string has enough space, then go ahead and write
	jr z,exec_c_wrte_sizefine
	ld (new_size),hl						;hl=new size of string
	call _enoughmem
	jp c,err_mem_file						;make sure there's enough free memory
	ld hl,temp_name-1
	call exec_c_wrte_create_new_dest		;then create a temp var
	ld hl,curr_name
	call exec_c_wrte_set_src_copy			;and copy the data to it
	ld hl,curr_name							;now delete the original var
	call exec_c_wrte_del
	ld hl,curr_name							;create a new var with the same name
	call exec_c_wrte_create_new_dest
	ld hl,temp_name-1						;copy the contents of the temp var to it
	call exec_c_wrte_set_src_copy
	ld hl,temp_name-1						;and delete the temp var
	call exec_c_wrte_del
exec_c_wrte_sizefine:
	ld hl,curr_name							;find the position in the string to write to
	call exec_c_wrte_get_var_start
	push af
	push hl
	ld de,(file_pos)
	ld hl,data_seg_len
	call mult16
	pop de
	pop af
	add hl,de
	adc a,0
;	push af
;	push hl
;	call _load_ram_ahl
;	ld a,(curr_var_type)
;	ld (hl),a								;and write the type byte
;	pop hl
;	pop af
;	call _inc_ptr_ahl
	ld b,a
	ld a,(curr_var_type)
	ld c,a
	ld a,b
	call _writeb_inc_ahl

	call _set_abs_dest_addr					;then the data!!!
	pop bc
	ld h,b
	ld l,c
	xor a
	call _set_mm_num_bytes
	pop hl
	call conv_to_abs_addr
	call _set_abs_src_addr
	call _mm_ldir
	jp _ram_page_1							;reset the ram page after the copy

.db 44
.dw 221

exec_c_read:
	call expect_num							;get the position to read from
	ld a,h									;make sure it isn't out of range,
	or a
	jp nz,err_argrange
	ld (file_pos),hl						;then save it
	call expect_varname						;get the var which will be holding the result
	ld (curr_var),de						;and save that too
	ld (var_type),a							;along with the vartype
	ld hl,curr_name							;now check if the file position is out of range
	rst 20h
	rst 10h
	push bc
	push de
	call _data_size_to_de
	ex de,hl
	ld de,data_seg_len
	call div16
	ld de,(file_pos)						;hl=max position in file, de=required position
	inc e
	call cphlde
	jr c,exec_c_read_blankX					;if input past end of file, return blank var
;	ld hl,(file_pos)						;if not, find the correct position in the file to read from
;	ld de,data_seg_len
	dec e
	ld hl,data_seg_len

	call mult16
	pop de
	pop af
	add hl,de
	adc a,0									;ahl=read pos
	call _ahl_plus_2
	push af
	push hl
	call _load_ram_ahl						;get the type of the var saved at that position
	ld a,(var_type)							;make sure it matches the type of the var we're reading to
	cp (hl)
	jp nz,err_vartype						;if it doesn't, show error
	pop hl
	pop af
	call _inc_ptr_ahl						;ahl=start of data
	call _set_abs_src_addr					;set that as source for copy
	ld a,(var_type)							;now branch depending on the vartype
	cp int_char
	jr z,exec_c_read_int
	cp array_char
	jr z,exec_c_read_array
exec_c_read_string:
;	call _ram_page_1
	ld hl,(curr_var)						;get the address of the string var where the output is going to be stored
	call conv_to_abs_addr					;convert that to an abs address
	call _set_abs_dest_addr					;to set it as the destination for the copy
	ld hl,str_max_len						;setting the length of the copy is easy - we know the max length for a string var already
	xor a
	call _set_mm_num_bytes
	jr exec_c_read_go
exec_c_read_array:
	call _load_abs_src_addr
	call _load_ram_ahl
	ld l,(hl)								;get the length of the array in the string
	ld a,l
	push af
	ld h,0
	add hl,hl
	inc hl
	xor a
	call _set_mm_num_bytes					;use that to set the copy length
	call _ram_page_1
	ld b,h
	ld c,l
	call alloc_mem							;and to create a new array var
	pop af
	ld (hl),a
	inc hl
	call conv_to_abs_addr
	call _set_abs_dest_addr					;and store its address
	ld de,(curr_var)
	call set_var
	jr exec_c_read_go
exec_c_read_blankX:
	jr exec_c_read_blank
exec_c_read_int:
	ld hl,(curr_var)
	call conv_to_abs_addr
	call _set_abs_dest_addr
	ld hl,2
	xor a
	call _set_mm_num_bytes
exec_c_read_go:
	call _mm_ldir							;finally, copy the actual data!!!
	jp _ram_page_1							;restore settings
exec_c_read_blank:
	pop hl
	pop hl
	call _ram_page_1						;if we have to return a blank var,
	ld a,(var_type)							;check what type of var it is
	cp int_char
	jr z,exec_c_read_blank_int
	cp array_char
	jr z,exec_c_read_blank_array
exec_c_read_blank_str:
	ld hl,(curr_var)
	ld (hl),0								;if it's a string, put a terminating char at the beginning
	ret
exec_c_read_blank_array:
	ld de,(curr_var)						;if it's an array, set the length to 0
	ld hl,blank_data
	jp set_var
exec_c_read_blank_int:
	ld hl,0									;if it's an int, just set it to 0
	ld de,(curr_var)
	jp set_var

.db 29
.dw 63

exec_c_polr:
	call expect_num							;first get the length of the vector
	ld (temp_buf),hl
	call expect_num							;then the angle
;	call is_hl_neg							;(compensate for negative angles)
;	jr c,exec_c_polr_skip
;	ld hl,360
;	add hl,de
;exec_c_polr_skip:
;	ld de,361								;(and make sure it's in range)
;	call cphlde
;	jp nc,err_argrange
;	ld de,trig_precision					;adjust the precision for the lookup table
;	call div16
;	add hl,hl								;*2
;	add hl,hl								;*4 (4 bytes for every entry)
;	ld de,trig_table
;	add hl,de								;get entry in trig table
;	ld e,(hl)								;first get the cos value...
;	inc hl
;	ld d,(hl)
;	inc hl
;	push hl
;	ld hl,(temp_buf)
	push hl
	call get_cos

	ld de,(temp_buf)
	call mult_num							;hl=r*(cos [theta]*1000)
	ld de,1000
	call div_num							;hl=r*cos [theta]
	push hl
	call expect_intname						;now get the var to store to the x pos
	pop hl
	call set_var_warray						;and store it
	pop hl
;	ld e,(hl)								;now get the sin value
;	inc hl
;	ld d,(hl)
;	ld hl,(temp_buf)
	ld de,-90
	add hl,de
	call get_cos

	ld de,(temp_buf)
	call mult_num							;hl=r*(sin [theta]*1000)
	ld de,1000
	call div_num							;hl=r*sin [theta]
	push hl
	call expect_intname						;and store that to the y var
	pop hl
	jp set_var_warray						;...that wasn't so hard...

.db 27
.dw 12

exec_c_rset:
	ld hl,func_stack						;first reset the function stack
	ld (func_sp),hl
	call clr_free_mem						;then reset all vars too
	jp exec_reset

.db 31
.dw 46

exec_c_subs:
	call expect_str							;get the string to separate
	push hl
	call expect_num							;now the starting position
	push hl
	call expect_num							;and the ending position
	pop de
	or a									;(make sure that the ending position is AFTER the starting position)
	sbc hl,de
	jp c,err_argrange
	inc hl
	push hl									;(and that the string isn't too long for a string var)
	ld bc,str_max_len+1
	or a
	sbc hl,bc
	pop hl
	jp nc,err_strlen
	ld b,h
	ld c,l
	pop hl
	add hl,de								;hl=starting address of segment, bc=length of segment
	push hl
	push bc
	call expect_strname						;get the name of the string var where the result will be stored
	pop bc
	pop hl									;hl=start of segment, bc=length of segment, de=destination
	ldir									;copy the string segment
	xor a
	ld (de),a								;and zero-terminate it
	ret

.db 32
.dw 16

exec_c_cint:
	call expect_str							;get the address of the string
	push hl
	call expect_intname						;and the name of the var where the result will be stored
	pop hl									;now parse the string!
	push de
	call str_to_num
	pop de									;finally, set the var to the parsed number
	jp set_var_warray

.db 30
.dw 23

exec_c_absl:
	call expect_num							;get the number
	call is_hl_neg
	jr c,exec_c_absl_set					;if it isn't negative, then it's fine as it is
	ld de,0
	ex de,hl
	or a
	sbc hl,de								;otherwise, invert it
exec_c_absl_set:
	push hl
	call expect_intname						;get the name of the var to store it in
	pop hl
	jp set_var_warray

.db 17
.dw 209

exec_c_inpt:
	call expect_x_pos						;first get the x pos of the prompt
	ld (_pencol),a
	call expect_y_pos						;then the y pos
	ld (_penrow),a							;and set the cursor
	call expect_str							;then get a pointer to the prompt text
	push hl
	call expect_color						;and find the color it should be drawn in
	pop hl
	or a
	jr z,exec_c_inpt_white
	dec a
	jr z,exec_c_inpt_lgray
	dec a
	jr z,exec_c_inpt_dgray
	dec a
	jr z,exec_c_inpt_black
exec_c_inpt_invert:
	ld bc,pixel_inv
	ld (pixel_op_patch),bc
	ld (Input_Str_Layer1),bc
	ld (Input_Str_Layer2),bc
	call draw_text_all
	jr exec_c_inpt_draw_end
exec_c_inpt_white:
	ld bc,pixel_off
	ld (pixel_op_patch),bc
	ld (Input_Str_Layer1),bc
	ld (Input_Str_Layer2),bc
	call draw_text_all
	jr exec_c_inpt_draw_end
exec_c_inpt_lgray:
	ld bc,pixel_on
	ld (pixel_op_patch),bc
	ld (Input_Str_Layer1),bc
	call draw_text_1
	ld bc,pixel_off
	ld (pixel_op_patch),bc
	ld (Input_Str_Layer2),bc
	call draw_text_2
	jr exec_c_inpt_draw_end
exec_c_inpt_dgray:
	ld bc,pixel_off
	ld (pixel_op_patch),bc
	ld (Input_Str_Layer1),bc
	call draw_text_1
	ld bc,pixel_on
	ld (pixel_op_patch),bc
	ld (Input_Str_Layer2),bc
	call draw_text_2
	jr exec_c_inpt_draw_end
exec_c_inpt_black:
	ld bc,pixel_on
	ld (pixel_op_patch),bc
	ld (Input_Str_Layer1),bc
	ld (Input_Str_Layer2),bc
	call draw_text_all
;	jr exec_c_inpt_draw_end
exec_c_inpt_draw_end:
	ld a,(_pencol)
	add a,b
	ld (_pencol),a
	call expect_varname						;after drawing the prompt, get the var where the input will be stored
	cp array_char
	jr z,exec_c_inpt_num
	cp int_char
	jr z,exec_c_inpt_num
	cp string_char
	jp nz,err_vartype
exec_c_inpt_str:
	ex de,hl								;if it's a string var, just store the string directly to it
	push hl
	call Input_Str
	pop de
	or a									;then make sure that it isn't too long
	sbc hl,de
	ld de,str_max_len+1
	call cphlde
	jp nc,err_strlen
	ret
exec_c_inpt_num:
	push de									;if it's a number,
	ld hl,temp_buf							;get the string in the temp buffer
	call Input_Str
	ld hl,temp_buf							;then convert it
	call str_to_num
	pop de
	jp set_var_warray						;and store it to the var

.db 38
.dw 74

exec_c_nmap:
	call expect_num							;get the x size
	push hl
	call expect_num							;and the y size
	push hl
	call expect_mapname						;then the address of the map to create
	ld (curr_var),de
	call get_var
	ld a,h
	or l
	jp nz,err_twovar_arry					;(make sure it hasn't been created already)
	pop de
	pop hl
	ld a,h									;make sure they're in range
	or a
	jp nz,err_maplen
	or l
	jp z,err_maplen
	ld a,d
	or a
	jp nz,err_maplen
	or e
	jp z,err_maplen
	ld b,l
	ld c,e
	push bc									;then save the sizes
	call mult16								;find the total length
	add hl,hl								;*2
	ld bc,4
	add hl,bc								;*2+4
	ld b,h
	ld c,l
	call alloc_mem							;so that we can allocate the memory for it
	pop bc
	push hl
	inc hl
	inc hl
	ld (hl),b								;set the size of the map from the user's parameters
	inc hl
	ld (hl),c
	pop hl
	ld de,(curr_var)
	jp set_var								;then set the map var to point to the newly-created map in memory

.db 39
.dw 74

tile_var	equ	temp_buf
tile_ctr	equ	temp_buf+2

exec_c_tile:
	call expect_mapname						;first get the name of the map where the solid tiles will be set
	call get_var
	ld a,h
	or l
	jp z,err_noarray
	inc hl
	inc hl
	ld c,(hl)
	inc hl
	ld b,(hl)								;then get the x and y sizes of the map
	inc hl
	ld (tile_var),hl
	ld h,b									;and multiply them to get the full length of the map data
	ld l,c
	call _htimesl
	ld (tile_ctr),hl
exec_c_tile_loop:
	ld hl,(parse_ptr)						;now check on the next argument
	ld a,(hl)
	cp eol_token							;if it's the end of the line,
	ret z									;then we're done
	call expect_bmpname						;otherwise, get the address of the bitmap to set as solid
	ld hl,(tile_var)						;then search for it in the current map
	ld bc,(tile_ctr)
exec_c_tile_search_loop:					;bc=number of tiles to search, hl=start of map data, de=tile to set as solid
	push hl
	call ldhlind							;and get the address of the tile in the map
	ld a,h
	and $0f
	or $a0
	ld h,a
	call cphlde								;is it one of the bitmaps that should be solid?
	pop hl
	inc hl
	jr nz,exec_c_tile_search_loop_dontset	;if it is, then go back and set the "solid/transparent" byte to 1
	ld a,$10
	or (hl)
	ld (hl),a
exec_c_tile_search_loop_dontset:
	inc hl									;move on to the next tile
	dec bc									;do this until all the tiles in the map have been tested
	ld a,b
	or c
	jr nz,exec_c_tile_search_loop
	jr exec_c_tile_loop						;when that's done, find the next bitmap that should be solid

.db 40
.dw 117

exec_c_cmap:
	call expect_mapname						;of course, the name of the MAP TO CHECK is the most important...
	call get_var
	ld a,h
	or l
	jp z,err_noarray
	ld a,(hl)								;get the map position (which is all we'll need for figuring out which element is touching...)
	ld (exec_c_cmap_x),a
	inc hl
	ld a,(hl)
	ld (exec_c_cmap_y),a
	call expect_sprname						;get the address of the sprite to check
	call get_spr							;and find its coordinates
	ld a,d									;(make sure it's been defined)
	or e
	jp z,err_nospr
	push bc
	call expect_arrayname					;then get the addresses of the arrays to store the positions
	push de
	call get_var
	pop de
	ld a,h
	or l
	jr nz,exec_c_cmap_skip_x
	push de
	ld bc,9									;and allocate a 4-element array for each of them
	call alloc_mem
	pop de
	ld (hl),4
	push hl
	call set_var
	pop hl
exec_c_cmap_skip_x:
	push hl
	call expect_arrayname
	push de
	call get_var
	pop de
	ld a,h
	or l
	jr nz,exec_c_cmap_skip_y
	push de
	ld bc,9
	call alloc_mem
	pop de
	ld (hl),4
	call set_var
	ex de,hl
exec_c_cmap_skip_y:
	ex de,hl
	pop hl									;de=address of y array, hl=address of x array
	inc hl
	inc de
	pop bc									;bc=sprite position
	call exec_c_cmap_get_tilenums			;first store the position of the top-left corner
	push bc
	ld a,b
	add a,7
	ld b,a
	call exec_c_cmap_get_tilenums			;then the top-right corner
	pop bc
	ld a,c
	add a,7
	ld c,a
	call exec_c_cmap_get_tilenums			;then the bottom-left
	ld a,b
	add a,7
	ld b,a
	jp exec_c_cmap_get_tilenums				;then the bottom-right

.db 41
.dw 107

tchk_map	equ	temp_buf

exec_c_tchk:
	call expect_mapname						;obviously, we'll need the address of the map var
	ld (tchk_map),de
	call get_var
	ld a,h
	or l
	jp z,err_noarray
	call expect_arrayname					;get address of x array
	call get_var
	ld a,h
	or l
	jp z,err_noarray
	push hl
	call expect_arrayname					;then the y array
	call get_var
	ld a,h
	or l
	jp z,err_noarray
	push hl
	call expect_intname						;then the int where the result will be stored
	ld (curr_var),de
	pop de
	pop hl
	ld a,(de)
	cp (hl)									;make sure the arrays have the same size
	jp nz,err_arrayindex	;??? find better error message
	ld b,a
	inc hl
	inc de									;hl=x array, de=y array, b=loop counter
exec_c_tchk_loop:
	push bc
	push hl
	push de
	ex de,hl
	ld c,(hl)
	inc hl
	ld b,(hl)
	ex de,hl
	call ldhlind							;get the positions from the arrays
	ld de,(tchk_map)						;and find the address of that element on the current map
	call find_map_element
	inc hl
	ld a,(hl)
	and $f0									;now for the big question - IS THE TILE SOLID???
	jr nz,exec_c_tchk_yes					;if it is, exit the loop and set the var
	pop de
	pop hl
	pop bc
	inc hl									;otherwise, move to the next array element
	inc hl
	inc de
	inc de
	djnz exec_c_tchk_loop
exec_c_tchk_no:								;if all of the positions have been checked and none are solid,
	ld hl,0									;then set the int to 0
	jr exec_c_tchk_end
exec_c_tchk_yes:
	pop de
	pop hl
	pop bc
	ld hl,1									;otherwise, if one of the tiles was solid, set the int to 1
exec_c_tchk_end:
	ld de,(curr_var)
	jp set_var_warray						;finally, set the result int

.db 42
.dw 88

dmap_x	equ	temp_buf
dmap_width	equ	temp_buf+1

exec_c_dmap:
	call expect_num							;get the position where the map will be displayed
	ld a,l
	push af
	call expect_num
	pop bc
	ld c,l									;bc=on-screen pos
	ld a,l
	push bc
	call expect_mapname						;now get the address of the map to draw
	pop bc
	call get_var
	ld a,h
	or l
	jp z,err_noarray
	ld (hl),b								;set its position,
	inc hl
	ld (hl),c
	inc hl
	ld a,(hl)								;and find its size
	ld d,a
	ld (dmap_width),a
	ld a,b
	ld (dmap_x),a
	inc hl
	ld e,(hl)
	inc hl
exec_c_dmap_loop:							;b,c=x,y pos	d,e=x,y size	hl=start of map pos
	push bc
	push de
	push hl
	call ldhlind							;get the bitmap address
	ld a,h
	or $a0
	ld h,a
	ld a,(hl)
	cp 8									;make sure that it's an 8x8 bitmap
	jp nz,err_map8
	inc hl
	call VeryFastGraySprite8				;then draw it!
	pop hl
	pop de
	pop bc
	inc hl
	inc hl									;point to the next bitmap,
	ld a,b
	add a,8
	ld b,a									;update the position,
	dec d									;and decrement the column counter
	jr nz,exec_c_dmap_loop					;if it's not done with this row, loop again
	ld a,(dmap_x)							;when at the end of the row, re-init the x position
	ld b,a
	ld a,(dmap_width)						;and the width (column counter)
	ld d,a
	ld a,c									;move to the next line
	add a,8
	ld c,a
	dec e									;and decrement the row counter
	jr nz,exec_c_dmap_loop					;again, if it's not at the end, move on to the next row
	ret

.db 0

.end