;MLC (Multi-platform Language for Calculators) by Epic Programming Studios
;TI-86 interpreter
;Written by Donald Straney (burntfuse) - straney@umbi.umd.edu

;Pre-tokenized program create/execute code


;structure of pre-tokenized program in file
;
;$01				1
;"MLC:"				4
;$D6				1
;[version string]	2
;[token list]		38
;[free1_mem_ptr]	2
;[free2_mem_ptr]	2
;[free3_mem_ptr]	2
;[free4_mem_ptr]	2
;[num_sprs]			1
;[start_addr]		2
;[bmp data len]		2
;(bmp data)
;(program)
;$FF				1


tok_prog:
	push hl
	rst 20h								;move the program's name to op1
	call _pushrealo1					;and save it
	pop hl
	call tok_part_1						;now tokenize the prog
	call tok_part_2
	ld hl,tok_header_str				;$ca00 is going to be used as a temp buffer for all the header data that needs to be written to the compressed file
	ld de,$ca00
	ld bc,6
	ldir								;so first copy the standard MLC "compressed program" header
	ld hl,version_str
	ld bc,2
	ldir								;...then the version string...
	ld hl,token_list
	ld bc,dyn_cmd_num
	ldir								;...then the token list...
	ld hl,free1_mem_ptr
	ld bc,8
	ldir								;...then the memory pointers...
	ld a,(num_sprites)
	ld (de),a
	inc de								;...then the number of sprites...
	ld hl,start_addr
	ld bc,2
	ldir								;...then the address of FNCT main...
	ld hl,(bmp_mem_ptr)
	ld bc,bmp_mem
	or a
	sbc hl,bc							;then check the number of bitmaps
	ex de,hl
	ld (hl),e							;(store that too)
	inc hl
	ld (hl),d
	inc hl
	ex de,hl
	ld a,h
	or l
	jr z,tok_prog_skip_bmp				;if there aren't any, then skip the next part
	ld b,h								;otherwise, copy the bitmap data to the buffer
	ld c,l
	ld hl,bmp_mem
	ldir
tok_prog_skip_bmp:
	ld hl,$ca00							;how much header data is really there?
	ex de,hl
	or a
	sbc hl,de
	push hl								;(save the result)
	push hl
	call _poprealo1
	call _pushrealo1
	rst 10h								;delete the original program (!)
	call _delvar
	call _poprealo1
	ld hl,(write_ptr)					;then find the size of the tokenized code,
	inc hl
	ld de,write_addr
	or a
	sbc hl,de
	pop de								;add that to the size of the header,
	add hl,de
	call _createprog					;and create a new program with the specified size to replace the old one
	ld a,b
	ex de,hl
	call _ahl_plus_2
	call _set_abs_dest_addr				;now dump all this crap to the file
	ld hl,$ca00							;first the header,
	xor a
	call _set_abs_src_addr
	pop hl
	call _mm_ldir_set_size
	ld a,1
	ld hl,write_addr-$8000
	call _set_abs_src_addr
	ld hl,(write_ptr)					;then the tokenized code
	inc hl
	ld de,write_addr
	xor a
	sbc hl,de
	call _mm_ldir_set_size
	jp prog_loop


exec_tok_prog:
	pop hl								;this routine ALWAYS needs to return 3 bytes after it was called, so might as well save space...
	inc hl
	inc hl
	inc hl
	push hl
	ld hl,copy_addr+6					;first check if the prog is the right version
	ld de,version_str
	ld a,(de)
	cp (hl)
	jp nz,err_version
	inc hl
	inc de
	ld a,(de)
	cp (hl)
	jp nz,err_version
	inc hl
	ld b,dyn_cmd_num
exec_tok_prog_loadtok_loop:				;ok, it is, now load the command handlers
	ld a,(hl)
	inc hl
	or a
	jr z,exec_tok_prog_loadtok_loop_end
	push bc
	ld b,a
	push hl
	ld hl,$0001
	call get_cmd_token_search
	pop hl
	pop bc
exec_tok_prog_loadtok_loop_end:
	djnz exec_tok_prog_loadtok_loop
	ld de,free1_mem_ptr					;then copy the memory pointers
	ld bc,8
	ldir
	ld a,(hl)							;...and the number of sprites...
	ld (num_sprites),a
	inc hl
	ld de,start_addr					;...and the address of FNCT main...
	ld bc,2
	ldir
	ld c,(hl)							;and the length of the stored bitmap data
	inc hl
	ld b,(hl)
	inc hl
	ld a,b
	or c
	jr z,exec_tok_prog_skip_bmp			;if there isn't any, we can skip it
	ld de,bmp_mem						;otherwise, copy it to the bitmap area
	ldir
exec_tok_prog_skip_bmp:
	ld de,write_addr					;finally, copy the tokenized code back to where it should be, so all the addresses work
	ld bc,end_copy_addr-copy_addr
	ldir
	ret

.end