;MLC (Multi-platform Language for Calculators) by Epic Programming Studios
;TI-86 interpreter
;Written by Donald Straney (burntfuse) - straney@umbi.umd.edu

;General routines


;input: none
;output: calculator turned off, a=0

turn_off_calc:
	ld a,$01
	out (3),a
	ld a,$56
	out (4),a
	halt
	ld a,$57
	out (4),a
	ld a,$0b
	out (3),a
	res onInterrupt,(iy+onflags)
	xor a
	ret


;input: hl is zero-terminated string
;output: a is length

strlen:
	ld b,0
strlen_loop:
	ld a,(hl)
	inc b
	or a
	jr nz,strlen_loop
	ld a,b
	dec a
	ret


;input: hl, de are strings to compare, lengths in b
;output: z if equal

strcmpb:
	push hl
	push de
strcmpb_loop:
	ld a,(de)
	cp (hl)
	jr nz,strcmpb_end
	inc hl
	inc de
	djnz strcmpb_loop
strcmpb_end:
	pop de
	pop hl
	ret


;input: hl, de are numbers to compare
;output: flags set

cphlde:
	push hl
	or a
	sbc hl,de
	pop hl
	ret


;input: hl is address
;output: hl=(hl)

ldhlind:
	ld a,(hl)
	inc hl
	ld h,(hl)
	ld l,a
	ret


;input: none
;output: _penXXX cursor moved to beginning of next line

_vnewline:
	push hl
	ld hl,_penrow
	ld a,(hl)
	add a,6
	ld (hl),a
	dec hl
_vnewline_patch	equ	$+1
	ld (hl),0
	pop hl
	ret


;input: hl is address (RAM page 1 loaded)
;output: ahl is abs address

conv_to_abs_addr:
	ld de,$c000							;is it on page 0?
	call cphlde
	jr c,conv_to_abs_addr_1
conv_to_abs_addr_0:
	xor a								;if so, no modifications needed!
	ret
conv_to_abs_addr_1:
	ld de,$8000							;otherwise, subtract $8000 from the address
	or a
	sbc hl,de
	ld a,1								;and set the RAM page
	ret


;input: bc is amount of memory (in bytes) to allocate
;output: hl is start of memory

alloc_mem:
	push de
	ld hl,(free1_mem_ptr)				;test the first memory area
	add hl,bc
	ld de,end_free1_mem+1
	call cphlde
	ld de,free1_mem_ptr
	jr c,alloc_mem_end					;if it can hold it, then use that one
	ld hl,(free2_mem_ptr)				;otherwise, try the next one
	add hl,bc
	ld de,end_free2_mem+1
	call cphlde
	ld de,free2_mem_ptr
	jr c,alloc_mem_end
	ld hl,(free3_mem_ptr)
	add hl,bc
	ld de,end_free3_mem+1
	call cphlde
	ld de,free3_mem_ptr
	jr c,alloc_mem_end
	ld hl,(free4_mem_ptr)
	add hl,bc
	ld de,end_free4_mem+1
	call cphlde
	ld de,free4_mem_ptr
	jp nc,err_mem						;if it can't fit in any of the areas, show error
alloc_mem_end:
	ex de,hl
	ld c,(hl)							;otherwise, update the appropriate free memory pointer
	ld (hl),e
	inc hl
	ld b,(hl)
	ld (hl),d
	ld h,b								;and return the address of the allocated memory
	ld l,c
	pop de
	ret


;input: hl is zero-terminated string to convert to number
;output: hl is converted number

str_to_num:
	ld de,temp_buf							;first copy number to temp buffer
	ld a,(hl)								;is the first char a '-'?
	cp Ldash
	jr nz,str_to_num_nominus				;if not, continue normally
	inc hl
	xor a									;otherwise, set to invert number
	jr $+4
str_to_num_nominus:
	ld a,$c9
	ld (str_to_num_patch),a
str_to_num_loop_1:
	ld a,(hl)
	or a
	jr z,str_to_num_end_loop_1
	sub L0
	cp 9+1									;are we at the end of the number yet?
	jp nc,err_cint
	ld (de),a								;if not, copy this digit to the buffer (as number, not char)
	inc de
	inc hl
	jr str_to_num_loop_1					;and loop
str_to_num_end_loop_1:
	ld hl,0									;(clear accumulator)
	ld (expr_sum),hl
	ld hl,temp_buf							;now convert number to hex
	ex de,hl
	or a
	sbc hl,de								;first get length
	ld b,l
	ld a,b									;check if it's too long
	cp num_max_len+1
	jp nc,err_numlen
	ld a,h
	or a
	jp nz,err_numlen
str_to_num_loop_2:
	ld a,(de)
	ld l,a
	ld h,0
	push bc
	dec b
	jr z,str_to_num_end_loop_3
str_to_num_loop_3:
	push bc
	call _hltimes10							;multiply digit by 10 correct number of times
	pop bc
	djnz str_to_num_loop_3
str_to_num_end_loop_3:
	ld bc,(expr_sum)						;then add to current sum
	add hl,bc
	ld (expr_sum),hl
	pop bc
	inc de									;and point to next digit
	djnz str_to_num_loop_2					;loop until all digits have been parsed
str_to_num_patch	equ	$
	nop
	dec hl
	ld a,h									;(if number is supposed to be negative, adjust it)
	cpl
	ld h,a
	ld a,l
	cpl
	ld l,a
	ret


;input: hl is number
;output: hl is pointer to start of converted string

format_num:
	call is_hl_neg
	jp c,FormatHL
	ld de,0
	ex de,hl							;if it's negative,
	or a
	sbc hl,de
	call FormatHL
	ld de,_op1+1
	ld bc,6
	ldir
	ld hl,_op1
	ld (hl),Ldash						;put a '-' in front of it
	ret


;input: a is standard color (0-3)
;output: d is layer 1 color, e is layer 2 color

get_color:
	inc a
	ret z
	dec a
	or a
	jp z,get_color_0
	dec a
	jp z,get_color_1
	dec a
	jp z,get_color_2
	dec a
	jp z,get_color_3
	jp err_argrange
get_color_0:
	ld de,0
	ret
get_color_1:
	ld de,$ff00
	ret
get_color_2:
	ld de,$00ff
	ret
get_color_3:
	ld de,$ffff
	ret


;input: prog name at (hl), destination abs address in bde
;output: file copied to dest address on RAM page 1, hl has prog length, c set if error

load_file:
	ld a,b
	ld (load_file_patch1),a
	ld (load_file_patch2),de
	rst 20h
	rst 10h								;find its address,
	ret c
	call _ex_ahl_bde
	call _ahl_plus_2_pg3				;get past the length bytes,
	call _set_abs_src_addr				;and set that as source address for copy
	ex de,hl
	call _data_size_to_de				;get the length of the file
	ex de,hl
	xor a
	call _set_mm_num_bytes				;and set that as length of copy
	push hl
load_file_patch1	equ	$+1
	ld a,0								;and set destination address for copy
load_file_patch2	equ	$+1
	ld hl,0
	call _set_abs_dest_addr
	call _mm_ldir						;now copy file
	pop hl
	xor a
	ret


;input: address of pointer in command table in hl, token of command in b
;output: handler loaded

get_cmd_token_search:
	push hl
	push bc
	ld hl,(token_list_ptr)				;write this token to the token list,
	ld (hl),b
	inc hl
	ld (token_list_ptr),hl
	ld hl,exec_str_name-1				;get the address of the handler string
	rst 20h
	rst 10h
	ex de,hl
	ld a,b
	call _ahl_plus_2
	call _abs_to_page_asic				;convert that to an address and page
	set 6,a
	out (5),a							;then load the pages it spans to search
	inc a
	out (6),a
	pop bc
	ld de,-$4000
	add hl,de
get_cmd_token_search_loop:
	ld a,(hl)							;get the token of the next handler
	or a								;(if we're at the end of the file, then this command must not have a handler)
	jp z,err_int
	inc hl
	ld e,(hl)
	inc hl
	ld d,(hl)
	inc hl
	cp b								;is it the one we want?
	jr z,get_cmd_token_load				;if it is, then copy it
	add hl,de							;otherwise, skip it
	jr get_cmd_token_search_loop
get_cmd_token_load:						;hl=source of handler
	ld b,d								;bc=size
	ld c,e
	ld de,(handler_write_ptr)			;de=destination
	push hl
	push de
	ex de,hl
	add hl,bc							;wait...where's the end of the handler going to be?
	ld (handler_write_ptr),hl			;(update write pointer while we're at it)
	ld de,handler_end_addr
	call cphlde							;is it PAST THE END OF THE ALLOWED AREA????
	jp nc,err_handlermem				;if not, then everything's fine
	pop de
	pop hl
	push de
	ldir								;copy it!!!!!! (wow, what a great instruction...)
	pop bc
	push af
	add a,a
	ld e,a
	ld d,0
	ld hl,exec_handler_table
	add hl,de
	ld a,$0d							;(switch back to the previous RAM/ROM pages)
	out (5),a
	call _ram_page_1	
	pop af
	ld (hl),c							;then show what address it's been loaded at in the handler address table
	inc hl
	ld (hl),b
	pop hl
	ld (hl),b							;and in the command table
	dec hl
	ld (hl),c
	ret


;input: none
;output: returns when no keys are being pressed

wait_for_no_key:
	call get_keycode
	or a
	jr nz,wait_for_no_key
	ret


;input: none
;output: returns when a key has been pressed, keycode in a

wait_for_a_key:
	call get_keycode
	or a
	jr z,wait_for_a_key
	ret


;input: none
;output: keycode in a

get_keycode:
	ld a,%10111111
	ld b,7
get_keycode_loop:
	ld e,a
	res 7,a
	ld d,b
	call get_key
	cp $ff
	jr nz,get_keycode_search
	ld a,e
	rra
	set 7,a
	ld b,d
	djnz get_keycode_loop
get_keycode_no:
	xor a
	ret
get_keycode_search:
	ld b,a
	ld hl,key_table
get_keycode_search_loop:
	ld a,(hl)
	or a
	ret z
	inc hl
	cp c
	jr nz,get_keycode_search_skip
	ld a,(hl)
	cp b
	jr z,get_keycode_search_found
get_keycode_search_skip:
	inc hl
	inc hl
	jr get_keycode_search_loop
get_keycode_search_found:
	inc hl
	ld a,(hl)
	ret


;input: key group in a
;output: scan code in a, key group in c

get_key:
	ld c,a
	out (1),a
	nop
	nop
	in a,(1)
	ret


;input: hl, de are numbers to multiply
;output: hl=hl*de

mult16:
	ld a,d
	or a
	jr z,mult16_de
	ld a,h
	or a
	jp nz,err_argrange
mult16_hl:
	ld b,l
	ex de,hl
	jr mult16_go
mult16_de:
	ld b,e
mult16_go:
	ld a,b
	or a
	jr z,mult16_0
	dec b
	ret z
	ld d,h
	ld e,l
mult16_loop:
	add hl,de
	djnz mult16_loop
	ret
mult16_0:
	ld hl,0
	ret


;input: hl is divisor, de is dividend
;output: hl=hl/de (rounded down if fraction)

div16:
	call cphlde
	jr z,div16_1
	jr c,div16_0
	ld a,d
	or a
	jr nz,div16_skip
	ld a,e
	dec a
	ret z
div16_skip:
	ld c,e
	ld b,d
	ex de,hl
	ld ix,0
	ld hl,0
div16_loop:
	add hl,bc
	jp c,div16_end
	inc ix
;	push hl
;	or a
;	sbc hl,de
;	pop hl
;	jp c,div16_loop
;	jr z,div16_end

	ld a,h
	cp d
;	jp c,div16_loop
	jr nz,div16_div_end
	ld a,l
	cp e
div16_div_end:
	jp c,div16_loop
	jr z,div16_end

	dec ix
div16_end:
	push ix
	pop hl
	ret
div16_1:
	ld hl,1
	ret
div16_0:
	ld hl,0
	ret


;input: none
;output: random number generator seeded

seed_random:
	ld a,r
	ld (_@RandomNumberSeed),a
	ld a,(_apdtimer)
	ld (_@RandomNumberSeed+1),a
	ret


;input: zero-terminated strings at (hl), (de)
;output: z if equal, nz if not equal

str_cmp:
	ld a,(de)
	ld b,(hl)
	cp b
	ret nz
	inc hl
	inc de
	or a
	ret z
	jr str_cmp


;input: hl is area to store string
;output: zero-terminated string entered by user stored to hl

Input_Str:
	xor a
	ld (Input_Str_Table),a
	ld (Input_Str_Start),hl
	ld (input_str_sp),sp
Input_Str_Loop:
	call Input_Str_Wait
	push hl
	ld hl,str_cursor
	ld bc,pixel_inv
	ld (pixel_op_patch),bc
	call draw_text_all
Input_Str_Key_Loop:
	call get_keycode
	or a
	jr z,Input_Str_Key_Loop
	ld c,a
	pop hl
	cp 95	;enter
	jr z,Input_Str_Enter
	cp 33	;delete
	jr z,Input_Str_Del
	cp 31	;alpha
	jp z,Input_Str_Alpha
	ld a,(_pencol)
	cp 112-5
	jr c,Input_Str_Find
	push hl
	jr Input_Str_Key_Loop
Input_Str_Find:
	ld de,input_str_keys_num
Input_Str_Table	equ	$+1
	ld b,0						;0=numbers, 1=letters
	djnz Input_Str_Find_Loop
	ld de,input_str_keys_alpha
Input_Str_Find_Loop:
	ld a,(de)
	or a
	jr z,Input_Str_Loop
	cp c
	jr z,Input_Str_Found
	inc de
	jr Input_Str_Find_Loop
Input_Str_Found:
	inc de
	ld a,(de)
	ld (hl),a
	inc hl
	ld a,(_pencol)
	push af
	push hl
	ld a,(de)
	ld hl,temp_buf
	ld (hl),a
	inc hl
	ld (hl),0
	ld hl,str_cursor
	ld bc,pixel_inv
	ld (pixel_op_patch),bc
	call draw_text_all
	ld hl,temp_buf
Input_Str_Layer1	equ	$+1
	ld bc,0
	ld (pixel_op_patch),bc
	call draw_text_1
Input_Str_Layer2	equ	$+1
	ld bc,0
	ld (pixel_op_patch),bc
	call draw_text_2
	ld a,(_pencol)
	add a,b
	ld (_pencol),a
	pop hl
	jp Input_Str_Loop
Input_Str_Enter:
	ld (hl),0
	ld sp,(input_str_sp)
	ret
Input_Str_Del:
Input_Str_Start	equ	$+1
	ld de,0
	call cphlde
	jp z,Input_Str_Loop
	push hl
	ld bc,pixel_inv
	ld (pixel_op_patch),bc
	ld hl,str_cursor
	call draw_text_all
	pop hl
	pop af
	ld (_pencol),a
	dec hl
	push hl
	ld a,(hl)
	ld hl,temp_buf
	ld (hl),a
	inc hl
	ld (hl),0
	dec hl
	ld bc,pixel_inv
	ld (pixel_op_patch),bc
	call draw_text_all
	pop hl
	jp Input_Str_Loop
Input_Str_Alpha:
	push hl
	ld a,(Input_Str_Table)
	xor 1
	ld (Input_Str_Table),a
	jp Input_Str_Key_Loop

Input_Str_Wait:
	ld de,key_delay
Input_Str_Wait_Loop:
	dec de
	ex (sp),hl
	ex (sp),hl
	ld a,d
	or e
	jr nz,Input_Str_Wait_Loop
	ret


;
; Lite86 II Decompression Routine
; written by Kirk Meyer
;
; NOTE: This must be called as a function
;       unless you modify the "ret z" line.
;
; Inputs:
;   HL -> the compressed data
;   DE -> where to store the output
;
; Outputs:
;   AF, BC, DE, HL trashed
;
Decompress:
 ld a,(hl)
 inc hl
 scf
 adc a,a
 .db $06
DecompressLoop:
 add a,a
 jr z,Decompress
 jr c,DecompressChunk
 ldi
 jr DecompressLoop
DecompressChunk:
 ld b,a
 ld c,$1F
 ld a,(hl)
 cp c
 ret z
 push bc 
 inc hl
 ld b,(hl)
 inc hl
 push hl
 ld l,b
 ld b,a
 or c
 rlca
 rlca
 rlca
 ld h,a
 ld a,b
 and c
 add a,3
 ld c,a
 ld b,0
 add hl,de
 ldir
 pop hl
 pop af
 jr DecompressLoop


;===========================================
; RandomNumber     [Assembly Coder's Zenith]
; out: A = psuedo random number
; modified: AF, BC    (14 bytes/62 t-states)
;===========================================
RandomNumber:
  ld a,r
  ld b,a
  ld a,($1234)
_@RandomNumberSeed =$-2
  ld c,a
  ld a,(bc)
  ld (_@RandomNumberSeed),a
  xor b
  sub b
  add a,c
  ret

.end